@@ -23,9 +23,9 @@ import org.apache.gluten.extension.injector.Injector
 
 class CHKafkaComponent extends Component {
   override def name(): String = "clickhouse-kafka"
-  override def buildInfo(): Component.BuildInfo =
-    Component.BuildInfo("ClickHouseKafka", "N/A", "N/A", "N/A")
+
   override def dependencies(): Seq[Class[_ <: Component]] = classOf[CHBackend] :: Nil
+
   override def injectRules(injector: Injector): Unit = {
     OffloadKafkaScan.inject(injector)
   }
