@@ -16,13 +16,13 @@
  */
 package org.apache.gluten.backendsapi.clickhouse
 
-import org.apache.gluten.config.GlutenConfig
+import org.apache.gluten.config.{ConfigRegistry, GlutenConfig}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.network.util.ByteUnit
 import org.apache.spark.sql.internal.SQLConf
 
-object CHConfig {
+object CHConfig extends ConfigRegistry {
   private[clickhouse] val BACKEND_NAME: String = "ch"
   private[clickhouse] val CONF_PREFIX: String = GlutenConfig.prefixOf(BACKEND_NAME)
   private val RUNTIME_SETTINGS: String = s"$CONF_PREFIX.runtime_settings"
@@ -59,8 +59,6 @@ object CHConfig {
 
   def get: CHConfig = new CHConfig(SQLConf.get)
 
-  import GlutenConfig._
-
   val ENABLE_ONEPIPELINE_MERGETREE_WRITE =
     buildConf(prefixOf("mergetree.write.pipeline"))
       .doc("Using one pipeline to write data to MergeTree table in Spark 3.5")
