@@ -24,18 +24,19 @@ import org.apache.gluten.expression._
 import org.apache.gluten.expression.ExpressionNames.MONOTONICALLY_INCREASING_ID
 import org.apache.gluten.extension.ExpressionExtensionTrait
 import org.apache.gluten.extension.columnar.heuristic.HeuristicTransform
+import org.apache.gluten.shuffle.NeedCustomColumnarBatchSerializer
 import org.apache.gluten.sql.shims.SparkShimLoader
 import org.apache.gluten.substrait.SubstraitContext
 import org.apache.gluten.substrait.expression.{ExpressionBuilder, ExpressionNode, WindowFunctionNode}
 import org.apache.gluten.utils.{CHJoinValidateUtil, UnknownJoinStrategy}
 import org.apache.gluten.vectorized.{BlockOutputStream, CHColumnarBatchSerializer, CHNativeBlock, CHStreamReader}
 
-import org.apache.spark.ShuffleDependency
+import org.apache.spark.{ShuffleDependency, SparkEnv}
 import org.apache.spark.internal.Logging
 import org.apache.spark.memory.SparkMemoryUtil
 import org.apache.spark.rdd.RDD
 import org.apache.spark.serializer.Serializer
-import org.apache.spark.shuffle.{GenShuffleWriterParameters, GlutenShuffleWriterWrapper, HashPartitioningWrapper}
+import org.apache.spark.shuffle.{GenShuffleReaderParameters, GenShuffleWriterParameters, GlutenShuffleReaderWrapper, GlutenShuffleWriterWrapper, HashPartitioningWrapper}
 import org.apache.spark.shuffle.utils.CHShuffleUtil
 import org.apache.spark.sql.catalyst.catalog.BucketSpec
 import org.apache.spark.sql.catalyst.catalog.CatalogTypes.TablePartitionSpec
@@ -440,6 +441,11 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
     CHShuffleUtil.genColumnarShuffleWriter(parameters)
   }
 
+  override def genColumnarShuffleReader[K, C](
+      parameters: GenShuffleReaderParameters[K, C]): GlutenShuffleReaderWrapper[K, C] = {
+    CHShuffleUtil.genColumnarShuffleReader(parameters)
+  }
+
   /**
    * Generate ColumnarBatchSerializer for ColumnarShuffleExchangeExec.
    *
@@ -453,15 +459,19 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
     val numOutputRows = metrics("numOutputRows")
     val dataSize = metrics("dataSize")
     val deserializationTime = metrics("deserializeTime")
-    if (GlutenConfig.get.isUseCelebornShuffleManager) {
-      val clazz = ClassUtils.getClass("org.apache.spark.shuffle.CHCelebornColumnarBatchSerializer")
-      val constructor =
-        clazz.getConstructor(classOf[SQLMetric], classOf[SQLMetric], classOf[SQLMetric])
-      constructor.newInstance(readBatchNumRows, numOutputRows, dataSize).asInstanceOf[Serializer]
-    } else if (GlutenConfig.get.isUseUniffleShuffleManager) {
-      throw new UnsupportedOperationException("temporarily uniffle not support ch ")
-    } else {
-      new CHColumnarBatchSerializer(readBatchNumRows, numOutputRows, dataSize, deserializationTime)
+    SparkEnv.get.shuffleManager match {
+      case serializer: NeedCustomColumnarBatchSerializer =>
+        val className = serializer.columnarBatchSerializerClass()
+        val clazz = ClassUtils.getClass(className)
+        val constructor =
+          clazz.getConstructor(classOf[SQLMetric], classOf[SQLMetric], classOf[SQLMetric])
+        constructor.newInstance(readBatchNumRows, numOutputRows, dataSize).asInstanceOf[Serializer]
+      case _ =>
+        new CHColumnarBatchSerializer(
+          readBatchNumRows,
+          numOutputRows,
+          dataSize,
+          deserializationTime)
     }
   }
 
@@ -504,7 +514,9 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
           val childWithAdapter = ColumnarCollapseTransformStages.wrapInputIteratorTransformer(child)
           WholeStageTransformer(
             ProjectExecTransformer(child.output ++ appendedProjections, childWithAdapter))(
-            ColumnarCollapseTransformStages.transformStageCounter.incrementAndGet()
+            ColumnarCollapseTransformStages
+              .getTransformStageCounter(childWithAdapter)
+              .incrementAndGet()
           )
         }
 
@@ -597,16 +609,11 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
   override def extraExpressionConverter(
       substraitExprName: String,
       expr: Expression,
-      attributeSeq: Seq[Attribute]): Option[ExpressionTransformer] = expr match {
-    case e if ExpressionExtensionTrait.findExpressionExtension(e.getClass).nonEmpty =>
-      // Use extended expression transformer to replace custom expression first
-      Some(
-        ExpressionExtensionTrait
-          .findExpressionExtension(e.getClass)
-          .get
-          .replaceWithExtensionExpressionTransformer(substraitExprName, e, attributeSeq))
-    case _ => None
-  }
+      attributeSeq: Seq[Attribute]): Option[ExpressionTransformer] =
+    // Use extended expression transformer to replace custom expression first
+    ExpressionExtensionTrait
+      .findExpressionExtension(expr.getClass)
+      .map(_.replaceWithExtensionExpressionTransformer(substraitExprName, expr, attributeSeq))
 
   override def genStringTranslateTransformer(
       substraitExprName: String,
@@ -636,6 +643,18 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
     CHTruncTimestampTransformer(substraitExprName, format, timestamp, timeZoneId, original)
   }
 
+  override def genToUnixTimestampTransformer(
+      substraitExprName: String,
+      timeExp: ExpressionTransformer,
+      format: ExpressionTransformer,
+      original: Expression): ExpressionTransformer = {
+    GenericExpressionTransformer(
+      substraitExprName,
+      Seq(timeExp, format),
+      original
+    )
+  }
+
   override def genDateDiffTransformer(
       substraitExprName: String,
       endDate: ExpressionTransformer,
@@ -832,10 +851,7 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
     }
   }
 
-  /** Clickhouse Backend only supports part of filters for parquet. */
-  override def postProcessPushDownFilter(
-      extraFilters: Seq[Expression],
-      sparkExecNode: LeafExecNode): Seq[Expression] = {
+  override def supportPushDownFilterToScan(sparkExecNode: LeafExecNode): Boolean = {
     // FIXME: DeltaMergeTreeFileFormat should not inherit from ParquetFileFormat.
     def isParquetFormat(fileFormat: FileFormat): Boolean = fileFormat match {
       case p: ParquetFileFormat if p.shortName().equals("parquet") => true
@@ -853,14 +869,28 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
     sparkExecNode match {
       case fileSourceScan: FileSourceScanExecTransformerBase
           if isParquetFormat(fileSourceScan.relation.fileFormat) =>
-        PushDownUtil.removeNotSupportPushDownFilters(
-          fileSourceScan.conf,
-          fileSourceScan.output,
-          fileSourceScan.dataFilters)
-      case _ => super.postProcessPushDownFilter(extraFilters, sparkExecNode)
+        false
+      case _ => true
     }
   }
 
+  /** Clickhouse Backend only supports part of filters for parquet. */
+  override def isSupportedScanFilter(filter: Expression, sparkExecNode: LeafExecNode): Boolean = {
+    // FIXME: DeltaMergeTreeFileFormat should not inherit from ParquetFileFormat.
+    def isParquetFormat(fileFormat: FileFormat): Boolean = fileFormat match {
+      case p: ParquetFileFormat if p.shortName().equals("parquet") => true
+      case _ => false
+    }
+
+    val isSupported = sparkExecNode match {
+      case fileSourceScan: FileSourceScanExecTransformerBase
+          if isParquetFormat(fileSourceScan.relation.fileFormat) =>
+        PushDownUtil.isSupportPushDownFilter(fileSourceScan.conf, fileSourceScan.output, filter)
+      case _ => true
+    }
+    isSupported && super.isSupportedScanFilter(filter, sparkExecNode)
+  }
+
   override def genGenerateTransformer(
       generator: Generator,
       requiredChildOutput: Seq[Attribute],
@@ -921,7 +951,7 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
       orderSpec: Seq[SortOrder],
       rankLikeFunction: Expression,
       limit: Int,
-      mode: WindowGroupLimitMode,
+      mode: GlutenWindowGroupLimitMode,
       child: SparkPlan): SparkPlan =
     CHWindowGroupLimitExecTransformer(
       partitionSpec,
@@ -948,15 +978,8 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
   override def genColumnarTailExec(limit: Int, child: SparkPlan): ColumnarCollectTailBaseExec =
     CHColumnarCollectTailExec(limit, child)
 
-  override def genColumnarRangeExec(
-      start: Long,
-      end: Long,
-      step: Long,
-      numSlices: Int,
-      numElements: BigInt,
-      outputAttributes: Seq[Attribute],
-      child: Seq[SparkPlan]): ColumnarRangeBaseExec =
-    CHRangeExecTransformer(start, end, step, numSlices, numElements, outputAttributes, child)
+  override def genColumnarRangeExec(rangeExec: RangeExec): ColumnarRangeBaseExec =
+    CHRangeExecTransformer(rangeExec.range)
 
   override def expressionFlattenSupported(expr: Expression): Boolean = expr match {
     case ca: FlattenedAnd => CHFlattenedExpression.supported(ca.name)
@@ -1026,4 +1049,13 @@ class CHSparkPlanExecApi extends SparkPlanExecApi with Logging {
       extract.get.last,
       original)
   }
+
+  override def genMonthsBetweenTransformer(
+      substraitExprName: String,
+      date1: ExpressionTransformer,
+      date2: ExpressionTransformer,
+      roundOff: ExpressionTransformer,
+      original: MonthsBetween): ExpressionTransformer = {
+    CHMonthsBetweenTransformer(substraitExprName, date1, date2, roundOff, original)
+  }
 }
