@@ -440,7 +440,7 @@ class GlutenClickHouseDecimalSuite extends ParquetSuite {
 
   test("Fix issue(6015) allow overflow when converting decimal to integer") {
     val sql = "select int(cast(id * 9999999999 as decimal(29, 2))) from range(10)"
-    runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
+    runQueryAndCompare(sql)(checkGlutenPlan[ProjectExecTransformer])
   }
 
   def testFromRandomBase(
