@@ -16,12 +16,5 @@
  */
 package org.apache.gluten.extensions;
 
-import java.util.Map;
-
 public class TestGlutenSystemFunctionPushDownInRowLevelOperations
-    extends TestGlutenSystemFunctionPushDownDQL {
-  public TestGlutenSystemFunctionPushDownInRowLevelOperations(
-      String catalogName, String implementation, Map<String, String> config) {
-    super(catalogName, implementation, config);
-  }
-}
+    extends TestGlutenSystemFunctionPushDownDQL {}
