@@ -16,12 +16,6 @@
  */
 package org.apache.gluten.source;
 
-import org.apache.iceberg.PlanningMode;
 import org.apache.iceberg.spark.source.TestIdentityPartitionData;
 
-public class TestGlutenIdentityPartitionData extends TestIdentityPartitionData {
-  public TestGlutenIdentityPartitionData(
-      String format, boolean vectorized, PlanningMode planningMode) {
-    super(format, vectorized, planningMode);
-  }
-}
+public class TestGlutenIdentityPartitionData extends TestIdentityPartitionData {}
