@@ -18,11 +18,4 @@ package org.apache.gluten.sql;
 
 import org.apache.iceberg.spark.sql.TestPartitionedWritesToWapBranch;
 
-import java.util.Map;
-
-public class TestGlutenPartitionedWritesToWapBranch extends TestPartitionedWritesToWapBranch {
-  public TestGlutenPartitionedWritesToWapBranch(
-      String catalogName, String implementation, Map<String, String> config) {
-    super(catalogName, implementation, config);
-  }
-}
+public class TestGlutenPartitionedWritesToWapBranch extends TestPartitionedWritesToWapBranch {}
