@@ -18,11 +18,4 @@ package org.apache.gluten.sql;
 
 import org.apache.iceberg.spark.sql.TestTimestampWithoutZone;
 
-import java.util.Map;
-
-public class TestGlutenTimestampWithoutZone extends TestTimestampWithoutZone {
-  public TestGlutenTimestampWithoutZone(
-      String catalogName, String implementation, Map<String, String> config) {
-    super(catalogName, implementation, config);
-  }
-}
+public class TestGlutenTimestampWithoutZone extends TestTimestampWithoutZone {}
