@@ -16,45 +16,23 @@
  */
 package org.apache.gluten.execution
 
-import org.apache.gluten.IcebergNestedFieldVisitor
-import org.apache.gluten.connector.write.{ColumnarBatchDataWriterFactory, IcebergDataWriteFactory}
-
 import org.apache.spark.sql.connector.write.Write
 import org.apache.spark.sql.execution.SparkPlan
 import org.apache.spark.sql.execution.datasources.v2._
-import org.apache.spark.sql.types.StructType
-
-import org.apache.iceberg.spark.source.IcebergWriteUtil
-import org.apache.iceberg.types.TypeUtil
 
 case class VeloxIcebergAppendDataExec(query: SparkPlan, refreshCache: () => Unit, write: Write)
-  extends IcebergAppendDataExec {
+  extends AbstractIcebergWriteExec {
 
-  override protected def withNewChildInternal(newChild: SparkPlan): IcebergAppendDataExec =
+  override protected def withNewChildInternal(newChild: SparkPlan): IcebergWriteExec =
     copy(query = newChild)
-
-  override def createFactory(schema: StructType): ColumnarBatchDataWriterFactory = {
-    val writeSchema = IcebergWriteUtil.getWriteSchema(write)
-    val nestedField = TypeUtil.visit(writeSchema, new IcebergNestedFieldVisitor)
-    IcebergDataWriteFactory(
-      schema,
-      getFileFormat(IcebergWriteUtil.getFileFormat(write)),
-      IcebergWriteUtil.getDirectory(write),
-      getCodec,
-      getPartitionSpec,
-      IcebergWriteUtil.getSortOrder(write),
-      nestedField
-    )
-  }
 }
 
 object VeloxIcebergAppendDataExec {
-  def apply(original: AppendDataExec): IcebergAppendDataExec = {
+  def apply(original: AppendDataExec): IcebergWriteExec = {
     VeloxIcebergAppendDataExec(
       original.query,
       original.refreshCache,
       original.write
     )
   }
-
 }
