@@ -32,7 +32,7 @@ import org.apache.spark.api.python.{ColumnarArrowEvalPythonExec, PullOutArrowEva
 import org.apache.spark.memory.SparkMemoryUtil
 import org.apache.spark.rdd.RDD
 import org.apache.spark.serializer.Serializer
-import org.apache.spark.shuffle.{GenShuffleWriterParameters, GlutenShuffleWriterWrapper}
+import org.apache.spark.shuffle.{GenShuffleReaderParameters, GenShuffleWriterParameters, GlutenShuffleReaderWrapper, GlutenShuffleWriterWrapper}
 import org.apache.spark.shuffle.utils.ShuffleUtil
 import org.apache.spark.sql.catalyst.catalog.BucketSpec
 import org.apache.spark.sql.catalyst.catalog.CatalogTypes.TablePartitionSpec
@@ -63,6 +63,8 @@ import javax.ws.rs.core.UriBuilder
 
 import java.util.Locale
 
+import scala.collection.JavaConverters._
+
 class VeloxSparkPlanExecApi extends SparkPlanExecApi {
 
   /** Transform GetArrayItem to Substrait. */
@@ -149,16 +151,6 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
         ExpressionMappings.expressionsMap(classOf[TryEval]),
         Seq(GenericExpressionTransformer(checkArithmeticExprName, Seq(left, right), original)),
         original)
-    } else if (
-      left.dataType.isInstanceOf[DecimalType] &&
-      right.dataType.isInstanceOf[DecimalType] &&
-      !SQLConf.get.decimalOperationsAllowPrecisionLoss
-    ) {
-      if (SparkShimLoader.getSparkShims.withAnsiEvalMode(original)) {
-        throw new GlutenNotSupportException(s"$substraitExprName with ansi mode is not supported")
-      }
-      val newName = substraitExprName + "_deny_precision_loss"
-      GenericExpressionTransformer(newName, Seq(left, right), original)
     } else if (SparkShimLoader.getSparkShims.withAnsiEvalMode(original)) {
       GenericExpressionTransformer(checkArithmeticExprName, Seq(left, right), original)
     } else {
@@ -166,6 +158,11 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
     }
   }
 
+  override def getDecimalArithmeticExprName(exprName: String): String = if (
+    !SQLConf.get.decimalOperationsAllowPrecisionLoss
+  ) { exprName + "_deny_precision_loss" }
+  else { exprName }
+
   /** Transform map_entries to Substrait. */
   override def genMapEntriesTransformer(
       substraitExprName: String,
@@ -205,6 +202,14 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
     }
   }
 
+  override def genArraySortTransformer(
+      substraitExprName: String,
+      argument: ExpressionTransformer,
+      function: ExpressionTransformer,
+      expr: ArraySort): ExpressionTransformer = {
+    GenericExpressionTransformer(substraitExprName, Seq(argument, function), expr)
+  }
+
   /** Transform array exists to Substrait */
   override def genArrayExistsTransformer(
       substraitExprName: String,
@@ -365,7 +370,15 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
 
     val newShuffle = shuffle.outputPartitioning match {
       case HashPartitioning(exprs, _) =>
-        val hashExpr = new Murmur3Hash(exprs)
+        val hashExpr = if (exprs.isEmpty) {
+          // In Spark, a hash expression with empty input is not resolvable and an
+          // `WRONG_NUM_ARGS.WITHOUT_SUGGESTION` error will be reported when validating the project
+          // transformer. So we directly return the seed here, which is the intended hashed value
+          // for empty input given Spark's murmur3 hash logic.
+          Literal(new Murmur3Hash(Nil).seed, IntegerType)
+        } else {
+          new Murmur3Hash(exprs)
+        }
         val projectList = Seq(Alias(hashExpr, "hash_partition_key")()) ++ child.output
         val projectTransformer = ProjectExecTransformer(projectList, child)
         val validationResult = projectTransformer.doValidate()
@@ -590,6 +603,11 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
     ShuffleUtil.genColumnarShuffleWriter(parameters)
   }
 
+  override def genColumnarShuffleReader[K, C](
+      parameters: GenShuffleReaderParameters[K, C]): GlutenShuffleReaderWrapper[K, C] = {
+    ShuffleUtil.genColumnarShuffleReader(parameters)
+  }
+
   override def createColumnarWriteFilesExec(
       child: WriteFilesExecTransformer,
       noop: SparkPlan,
@@ -662,26 +680,32 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
       dataSize: SQLMetric): BuildSideRelation = {
     val useOffheapBroadcastBuildRelation =
       VeloxConfig.get.enableBroadcastBuildRelationInOffheap
-    val serialized: Array[ColumnarBatchSerializeResult] = child
+    val serialized: Seq[ColumnarBatchSerializeResult] = child
       .executeColumnar()
       .mapPartitions(itr => Iterator(BroadcastUtils.serializeStream(itr)))
-      .filter(_.getNumRows != 0)
+      .filter(_.numRows != 0)
       .collect
-    val rawSize = serialized.flatMap(_.getSerialized.map(_.length.toLong)).sum
+    val rawSize = serialized.map(_.sizeInBytes()).sum
     if (rawSize >= GlutenConfig.get.maxBroadcastTableSize) {
       throw new SparkException(
         "Cannot broadcast the table that is larger than " +
           s"${SparkMemoryUtil.bytesToString(GlutenConfig.get.maxBroadcastTableSize)}: " +
           s"${SparkMemoryUtil.bytesToString(rawSize)}")
     }
-    numOutputRows += serialized.map(_.getNumRows).sum
+    numOutputRows += serialized.map(_.numRows).sum
     dataSize += rawSize
     if (useOffheapBroadcastBuildRelation) {
       TaskResources.runUnsafe {
-        UnsafeColumnarBuildSideRelation(child.output, serialized.flatMap(_.getSerialized), mode)
+        UnsafeColumnarBuildSideRelation(
+          child.output,
+          serialized.flatMap(_.offHeapData().asScala),
+          mode)
       }
     } else {
-      ColumnarBuildSideRelation(child.output, serialized.flatMap(_.getSerialized), mode)
+      ColumnarBuildSideRelation(
+        child.output,
+        serialized.flatMap(_.onHeapData().asScala).toArray,
+        mode)
     }
   }
 
@@ -817,6 +841,14 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
         ExpressionNames.TO_JSON,
         ToJsonRestrictions.NOT_SUPPORT_WITH_OPTIONS)
     }
+    if (
+      !SQLConf.get.caseSensitiveAnalysis &&
+      ExpressionUtils.hasUppercaseStructFieldName(child.dataType)
+    ) {
+      GlutenExceptionUtil.throwsNotFullySupported(
+        ExpressionNames.TO_JSON,
+        ToJsonRestrictions.NOT_SUPPORT_UPPERCASE_STRUCT)
+    }
     ToJsonTransformer(substraitExprName, child, expr)
   }
 
@@ -996,15 +1028,8 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
       offset: Int): ColumnarCollectLimitBaseExec =
     ColumnarCollectLimitExec(limit, child, offset)
 
-  override def genColumnarRangeExec(
-      start: Long,
-      end: Long,
-      step: Long,
-      numSlices: Int,
-      numElements: BigInt,
-      outputAttributes: Seq[Attribute],
-      child: Seq[SparkPlan]): ColumnarRangeBaseExec =
-    ColumnarRangeExec(start, end, step, numSlices, numElements, outputAttributes, child)
+  override def genColumnarRangeExec(rangeExec: RangeExec): ColumnarRangeBaseExec =
+    ColumnarRangeExec(rangeExec.range)
 
   override def genColumnarTailExec(limit: Int, child: SparkPlan): ColumnarCollectTailBaseExec =
     ColumnarCollectTailExec(limit, child)
@@ -1040,4 +1065,31 @@ class VeloxSparkPlanExecApi extends SparkPlanExecApi {
     }
     TimestampDiffTransformer(substraitExprName, extract.get, left, right, original)
   }
+
+  override def genToUnixTimestampTransformer(
+      substraitExprName: String,
+      timeExp: ExpressionTransformer,
+      format: ExpressionTransformer,
+      original: Expression): ExpressionTransformer = {
+    ToUnixTimestampTransformer(substraitExprName, timeExp, format, original)
+  }
+
+  override def genMonthsBetweenTransformer(
+      substraitExprName: String,
+      date1: ExpressionTransformer,
+      date2: ExpressionTransformer,
+      roundOff: ExpressionTransformer,
+      original: MonthsBetween): ExpressionTransformer = {
+    MonthsBetweenTransformer(substraitExprName, date1, date2, roundOff, original)
+  }
+
+  override def getErrorMessage(raiseError: RaiseError): Expression = {
+    SparkShimLoader.getSparkShims.getErrorMessage(raiseError) match {
+      case Some(msg) => msg
+      case None =>
+        GlutenExceptionUtil.throwsNotFullySupported(
+          ExpressionNames.RAISE_ERROR,
+          RaiseErrorRestrictions.ONLY_SUPPORT_ERROR_MESSAGE)
+    }
+  }
 }
