@@ -34,6 +34,7 @@ import org.apache.spark.sql.types.StructType
 import org.apache.spark.sql.vectorized.ColumnarBatch
 import org.apache.spark.task.TaskResources
 
+import scala.collection.JavaConverters._
 import scala.collection.mutable.ArrayBuffer
 
 // Utility methods to convert Vanilla broadcast relations from/to Velox broadcast relations.
@@ -93,58 +94,44 @@ object BroadcastUtils {
       schema: StructType,
       from: Broadcast[F],
       fn: Iterator[InternalRow] => Iterator[ColumnarBatch]): Broadcast[T] = {
-    val useOffheapBuildRelation = VeloxConfig.get.enableBroadcastBuildRelationInOffheap
+
+    def batchIterationToRelation(batchItr: () => Iterator[ColumnarBatch]): BuildSideRelation = {
+      TaskResources.runUnsafe {
+        serializeStream(batchItr()) match {
+          case ColumnarBatchSerializeResult.EMPTY =>
+            ColumnarBuildSideRelation(
+              SparkShimLoader.getSparkShims.attributesFromStruct(schema),
+              Array[Array[Byte]](),
+              mode)
+          case result: ColumnarBatchSerializeResult =>
+            if (result.isOffHeap) {
+              UnsafeColumnarBuildSideRelation(
+                SparkShimLoader.getSparkShims.attributesFromStruct(schema),
+                result.offHeapData().asScala.toSeq,
+                mode)
+            } else {
+              ColumnarBuildSideRelation(
+                SparkShimLoader.getSparkShims.attributesFromStruct(schema),
+                result.onHeapData().asScala.toArray,
+                mode)
+            }
+        }
+      }
+    }
+
     mode match {
       case HashedRelationBroadcastMode(_, _) =>
         // HashedRelation to ColumnarBuildSideRelation.
         val fromBroadcast = from.asInstanceOf[Broadcast[HashedRelation]]
         val fromRelation = fromBroadcast.value.asReadOnlyCopy()
-        val toRelation = TaskResources.runUnsafe {
-          val batchItr: Iterator[ColumnarBatch] = fn(reconstructRows(fromRelation))
-          val serialized: Array[Array[Byte]] = serializeStream(batchItr) match {
-            case ColumnarBatchSerializeResult.EMPTY =>
-              Array()
-            case result: ColumnarBatchSerializeResult =>
-              result.getSerialized
-          }
-          if (useOffheapBuildRelation) {
-            UnsafeColumnarBuildSideRelation(
-              SparkShimLoader.getSparkShims.attributesFromStruct(schema),
-              serialized,
-              mode)
-          } else {
-            ColumnarBuildSideRelation(
-              SparkShimLoader.getSparkShims.attributesFromStruct(schema),
-              serialized,
-              mode)
-          }
-        }
+        val toRelation = batchIterationToRelation(() => fn(reconstructRows(fromRelation)))
         // Rebroadcast Velox relation.
         context.broadcast(toRelation).asInstanceOf[Broadcast[T]]
       case IdentityBroadcastMode =>
         // Array[InternalRow] to ColumnarBuildSideRelation.
         val fromBroadcast = from.asInstanceOf[Broadcast[Array[InternalRow]]]
         val fromRelation = fromBroadcast.value
-        val toRelation = TaskResources.runUnsafe {
-          val batchItr: Iterator[ColumnarBatch] = fn(fromRelation.iterator)
-          val serialized: Array[Array[Byte]] = serializeStream(batchItr) match {
-            case ColumnarBatchSerializeResult.EMPTY =>
-              Array()
-            case result: ColumnarBatchSerializeResult =>
-              result.getSerialized
-          }
-          if (useOffheapBuildRelation) {
-            UnsafeColumnarBuildSideRelation(
-              SparkShimLoader.getSparkShims.attributesFromStruct(schema),
-              serialized,
-              mode)
-          } else {
-            ColumnarBuildSideRelation(
-              SparkShimLoader.getSparkShims.attributesFromStruct(schema),
-              serialized,
-              mode)
-          }
-        }
+        val toRelation = batchIterationToRelation(() => fn(fromRelation.iterator))
         // Rebroadcast Velox relation.
         context.broadcast(toRelation).asInstanceOf[Broadcast[T]]
       case _ => throw new IllegalStateException("Unexpected broadcast mode: " + mode)
@@ -188,7 +175,12 @@ object BroadcastUtils {
         })
       .toArray
     if (values.nonEmpty) {
-      new ColumnarBatchSerializeResult(numRows, values)
+      val useOffheapBroadcastBuildRelation =
+        VeloxConfig.get.enableBroadcastBuildRelationInOffheap
+      new ColumnarBatchSerializeResult(
+        useOffheapBroadcastBuildRelation,
+        numRows,
+        values.toSeq.asJava)
     } else {
       ColumnarBatchSerializeResult.EMPTY
     }
