@@ -23,7 +23,7 @@ import org.apache.spark.SparkConf
 import org.apache.spark.sql.Row
 import org.apache.spark.sql.execution.{ColumnarToRowExec, SparkPlan}
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
-import org.apache.spark.sql.execution.columnar.InMemoryTableScanExec
+import org.apache.spark.sql.execution.columnar.{InMemoryTableScanExec, SparkCacheUtil}
 import org.apache.spark.sql.types.{LongType, Metadata, MetadataBuilder, StructType}
 import org.apache.spark.storage.StorageLevel
 
@@ -35,9 +35,17 @@ class VeloxColumnarCacheSuite extends VeloxWholeStageTransformerSuite with Adapt
 
   override def beforeAll(): Unit = {
     super.beforeAll()
+    // A common practice as well as in Spark tests, to clear the cache serializer
+    // in case it was already set as the default row-based serializer.
+    SparkCacheUtil.clearCacheSerializer()
     createTPCHNotNullTables()
   }
 
+  override protected def afterAll(): Unit = {
+    SparkCacheUtil.clearCacheSerializer()
+    super.afterAll()
+  }
+
   override protected def sparkConf: SparkConf = {
     super.sparkConf
       .set("spark.shuffle.manager", "org.apache.spark.shuffle.sort.ColumnarShuffleManager")
@@ -102,7 +110,7 @@ class VeloxColumnarCacheSuite extends VeloxWholeStageTransformerSuite with Adapt
   }
 
   // See issue https://github.com/apache/incubator-gluten/issues/8497.
-  testWithMinSparkVersion("Input fallen back vanilla Spark columnar scan", "3.3") {
+  test("Input fallen back vanilla Spark columnar scan") {
     def withId(id: Int): Metadata =
       new MetadataBuilder().putLong("parquet.field.id", id).build()
 
