@@ -198,7 +198,7 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
       "select type1.short, int from type1" +
         " where type1.short = 1",
       false) {
-      checkGlutenOperatorMatch[BatchScanExecTransformer]
+      checkGlutenPlan[BatchScanExecTransformer]
     }
 
     // Validation: BatchScan Project Aggregate Expand Sort Limit
@@ -269,13 +269,13 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
                          | select date, int from type1 limit 100
                          |) where int != 0 limit 10;
                          |""".stripMargin) {
-      checkGlutenOperatorMatch[LimitExecTransformer]
+      checkGlutenPlan[LimitExecTransformer]
     }
 
     // Validation: Window.
     runQueryAndCompare(
       "select row_number() over (partition by date order by int) from type1 order by int, date") {
-      checkGlutenOperatorMatch[WindowExecTransformer]
+      checkGlutenPlan[WindowExecTransformer]
     }
 
     // Validation: BroadHashJoin.
@@ -283,7 +283,7 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
       runQueryAndCompare(
         "select type1.date from type1," +
           " type2 where type1.date = type2.date") {
-        checkGlutenOperatorMatch[BroadcastHashJoinExecTransformer]
+        checkGlutenPlan[BroadcastHashJoinExecTransformer]
       }
     }
 
@@ -294,7 +294,7 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
       runQueryAndCompare(
         "select type1.date from type1," +
           " type2 where type1.date = type2.date") {
-        checkGlutenOperatorMatch[ShuffledHashJoinExecTransformer]
+        checkGlutenPlan[ShuffledHashJoinExecTransformer]
       }
     }
 
@@ -304,7 +304,7 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
         runQueryAndCompare(
           "select type1.date from type1," +
             " type2 where type1.date = type2.date") {
-          checkGlutenOperatorMatch[SortMergeJoinExecTransformer]
+          checkGlutenPlan[SortMergeJoinExecTransformer]
         }
       }
     }
@@ -333,7 +333,7 @@ class VeloxParquetDataTypeValidationSuite extends VeloxWholeStageTransformerSuit
     withSQLConf(("spark.gluten.sql.complexType.scan.fallback.enabled", "false")) {
       // Validation: BatchScan.
       runQueryAndCompare("select array from type1") {
-        checkGlutenOperatorMatch[BatchScanExecTransformer]
+        checkGlutenPlan[BatchScanExecTransformer]
       }
 
       // Validation: BatchScan Project Aggregate Expand Sort Limit
