@@ -73,6 +73,7 @@ abstract class VeloxTPCHSuite extends VeloxTPCHTableSupport {
       // for unexpected blank
       .replaceAll("Scan parquet ", "Scan parquet")
       // Spark QueryStageExec will take it's id as argument, replace it with X
+      .replaceAll("Arguments: [0-9]+, [0-9]+, [0-9]+", "Arguments: X, X, X")
       .replaceAll("Arguments: [0-9]+, [0-9]+", "Arguments: X, X")
       .replaceAll("Arguments: [0-9]+", "Arguments: X")
       // mask PullOutPostProject and PullOutPreProject id
@@ -92,6 +93,7 @@ abstract class VeloxTPCHSuite extends VeloxTPCHTableSupport {
         case "32" => true
         case "33" => true
         case "34" => true
+        case "35" => true
         case _ => false
       }
     )
