@@ -21,6 +21,8 @@ import org.apache.gluten.execution.WholeStageTransformerSuite
 import org.apache.spark.SparkConf
 import org.apache.spark.api.python.ColumnarArrowEvalPythonExec
 import org.apache.spark.sql.IntegratedUDFTestUtils
+import org.apache.spark.sql.types.{DataType, LongType, StringType}
+import org.apache.spark.util.SparkVersionUtil
 
 class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
 
@@ -30,7 +32,10 @@ class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
 
   override protected val resourcePath: String = "/tpch-data-parquet"
   override protected val fileFormat: String = "parquet"
-  val pyarrowTestUDF = TestScalarPandasUDF(name = "pyarrowUDF")
+  private val pyarrowTestUDFString =
+    newTestScalarPandasUDF(name = "pyarrowUDF", returnType = Some(StringType))
+  private val pyarrowTestUDFLong =
+    newTestScalarPandasUDF(name = "pyarrowUDF", returnType = Some(LongType))
 
   override def sparkConf: SparkConf = {
     super.sparkConf
@@ -39,7 +44,8 @@ class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
       .set("spark.executor.cores", "1")
   }
 
-  test("arrow_udf test: without projection") {
+  // TODO: fix on spark-4.1
+  testWithMaxSparkVersion("arrow_udf test: without projection", "4.0") {
     lazy val base =
       Seq(("1", 1), ("1", 2), ("2", 1), ("2", 2), ("3", 1), ("3", 2), ("0", 1), ("3", 0))
         .toDF("a", "b")
@@ -54,12 +60,13 @@ class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
       ("3", "3")
     ).toDF("a", "p_a")
 
-    val df2 = base.select("a").withColumn("p_a", pyarrowTestUDF(base("a")))
-    checkSparkOperatorMatch[ColumnarArrowEvalPythonExec](df2)
+    val df2 = base.select("a").withColumn("p_a", pyarrowTestUDFString(base("a")))
+    checkSparkPlan[ColumnarArrowEvalPythonExec](df2)
     checkAnswer(df2, expected)
   }
 
-  test("arrow_udf test: with unrelated projection") {
+  // TODO: fix on spark-4.1
+  testWithMaxSparkVersion("arrow_udf test: with unrelated projection", "4.0") {
     lazy val base =
       Seq(("1", 1), ("1", 2), ("2", 1), ("2", 2), ("3", 1), ("3", 2), ("0", 1), ("3", 0))
         .toDF("a", "b")
@@ -74,12 +81,14 @@ class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
       ("3", 0, "3", 0)
     ).toDF("a", "b", "p_a", "d_b")
 
-    val df = base.withColumn("p_a", pyarrowTestUDF(base("a"))).withColumn("d_b", base("b") * 2)
-    checkSparkOperatorMatch[ColumnarArrowEvalPythonExec](df)
+    val df =
+      base.withColumn("p_a", pyarrowTestUDFString(base("a"))).withColumn("d_b", base("b") * 2)
+    checkSparkPlan[ColumnarArrowEvalPythonExec](df)
     checkAnswer(df, expected)
   }
 
-  test("arrow_udf test: with preprojection") {
+  // TODO: fix on spark-4.1
+  testWithMaxSparkVersion("arrow_udf test: with preprojection", "4.0") {
     lazy val base =
       Seq(("1", 1), ("1", 2), ("2", 1), ("2", 2), ("3", 1), ("3", 2), ("0", 1), ("3", 0))
         .toDF("a", "b")
@@ -95,8 +104,23 @@ class ArrowEvalPythonExecSuite extends WholeStageTransformerSuite {
     ).toDF("a", "b", "d_b", "p_a", "p_b")
     val df = base
       .withColumn("d_b", base("b") * 2)
-      .withColumn("p_a", pyarrowTestUDF(base("a")))
-      .withColumn("p_b", pyarrowTestUDF(base("b") * 2))
+      .withColumn("p_a", pyarrowTestUDFString(base("a")))
+      .withColumn("p_b", pyarrowTestUDFLong(base("b") * 2))
     checkAnswer(df, expected)
   }
+
+  private def newTestScalarPandasUDF(
+      name: String,
+      returnType: Option[DataType] = None): TestScalarPandasUDF = {
+    if (SparkVersionUtil.gteSpark40) {
+      // After https://github.com/apache/spark/pull/42864 which landed in Spark 4.0, the return
+      // type of the UDF must be explicitly specified when creating the UDF instance with column
+      // expressions as parameter.
+      classOf[TestScalarPandasUDF]
+        .getConstructor(classOf[String], classOf[Option[DataType]])
+        .newInstance(name, returnType)
+    } else {
+      TestScalarPandasUDF(name)
+    }
+  }
 }
