@@ -14,21 +14,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-package org.apache.spark.memory;
+package org.apache.spark.memory
 
 import org.apache.gluten.config.GlutenCoreConfig
-import org.apache.gluten.exception.GlutenException
 import org.apache.gluten.memory.memtarget.{Spillers, TreeMemoryTarget}
+import org.apache.gluten.memory.memtarget.ThrowOnOomMemoryTarget.OutOfMemoryException
 import org.apache.gluten.memory.memtarget.spark.TreeMemoryConsumers
 
-import org.apache.spark.TaskContext
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.task.TaskResources
 
 import org.scalatest.BeforeAndAfterAll
 import org.scalatest.funsuite.AnyFunSuite
 
-import java.util.Collections;
+import java.util.Collections
 
 class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
 
@@ -42,7 +41,7 @@ class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
   test("Sanity") {
     TaskResources.runUnsafe {
       val factory =
-        TreeMemoryConsumers.factory(TaskContext.get().taskMemoryManager(), MemoryMode.OFF_HEAP)
+        TreeMemoryConsumers.factory(MemoryMode.OFF_HEAP)
       val consumer =
         factory
           .legacyRoot()
@@ -54,18 +53,18 @@ class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
       assert(consumer.borrow(300) == 300)
       GlobalOffHeapMemory.acquire(50)
       GlobalOffHeapMemory.acquire(40)
-      assertThrows[GlutenException](GlobalOffHeapMemory.acquire(30))
-      assertThrows[GlutenException](GlobalOffHeapMemory.acquire(11))
+      assertThrows[OutOfMemoryException](GlobalOffHeapMemory.acquire(30))
+      assertThrows[OutOfMemoryException](GlobalOffHeapMemory.acquire(11))
       GlobalOffHeapMemory.acquire(10)
       GlobalOffHeapMemory.acquire(0)
-      assertThrows[GlutenException](GlobalOffHeapMemory.acquire(1))
+      assertThrows[OutOfMemoryException](GlobalOffHeapMemory.acquire(1))
     }
   }
 
   test("Task OOM by global occupation") {
     TaskResources.runUnsafe {
       val factory =
-        TreeMemoryConsumers.factory(TaskContext.get().taskMemoryManager(), MemoryMode.OFF_HEAP)
+        TreeMemoryConsumers.factory(MemoryMode.OFF_HEAP)
       val consumer =
         factory
           .legacyRoot()
@@ -84,7 +83,7 @@ class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
   test("Release global") {
     TaskResources.runUnsafe {
       val factory =
-        TreeMemoryConsumers.factory(TaskContext.get().taskMemoryManager(), MemoryMode.OFF_HEAP)
+        TreeMemoryConsumers.factory(MemoryMode.OFF_HEAP)
       val consumer =
         factory
           .legacyRoot()
@@ -103,7 +102,7 @@ class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
   test("Release task") {
     TaskResources.runUnsafe {
       val factory =
-        TreeMemoryConsumers.factory(TaskContext.get().taskMemoryManager(), MemoryMode.OFF_HEAP)
+        TreeMemoryConsumers.factory(MemoryMode.OFF_HEAP)
       val consumer =
         factory
           .legacyRoot()
@@ -113,7 +112,7 @@ class GlobalOffHeapMemorySuite extends AnyFunSuite with BeforeAndAfterAll {
             Spillers.NOOP,
             Collections.emptyMap())
       assert(consumer.borrow(300) == 300)
-      assertThrows[GlutenException](GlobalOffHeapMemory.acquire(200))
+      assertThrows[OutOfMemoryException](GlobalOffHeapMemory.acquire(200))
       assert(consumer.repay(100) == 100)
       GlobalOffHeapMemory.acquire(200)
     }
