@@ -44,7 +44,7 @@ class VeloxParquetReadSuite extends VeloxWholeStageTransformerSuite {
               val df = spark.read.parquet(file.getAbsolutePath)
               df.createOrReplaceTempView("test_table")
               runQueryAndCompare("select * from test_table") {
-                checkGlutenOperatorMatch[BasicScanExecTransformer]
+                checkGlutenPlan[BasicScanExecTransformer]
               }
             }
         }
