@@ -27,6 +27,7 @@
 #include <Common/ThreadStatus.h>
 #include <Common/formatReadable.h>
 #include <Common/logger_useful.h>
+#include <Parser/LocalExecutor.h>
 
 namespace DB
 {
@@ -59,6 +60,7 @@ DB::ContextMutablePtr QueryContext::globalMutableContext()
 {
     return Data::global_context;
 }
+
 void QueryContext::resetGlobal()
 {
     if (Data::global_context)
@@ -69,6 +71,23 @@ void QueryContext::resetGlobal()
     Data::shared_context.reset();
 }
 
+void QueryContext::reset()
+{
+    {
+         std::lock_guard<std::mutex> lock(local_executor_mutex_);
+        for (auto it = active_executors_.begin(); it != active_executors_.end(); ++it)
+        {
+            auto * executor = *it;
+            executor->cancel();
+            LOG_ERROR(logger_, "Not closed LocalExecutor({}):\n{}", reinterpret_cast<intptr_t>(executor), executor->dumpPipeline());
+            delete executor;
+        }
+        active_executors_.clear();
+    }
+    query_map_.clear();
+    QueryContext::resetGlobal();
+}
+
 DB::ContextMutablePtr QueryContext::createGlobal()
 {
     assert(Data::shared_context.get() == nullptr);
@@ -195,14 +214,20 @@ void QueryContext::finalizeQuery(int64_t id)
 size_t currentThreadGroupMemoryUsage()
 {
     if (!CurrentThread::getGroup())
-        throw DB::Exception(ErrorCodes::LOGICAL_ERROR, "Thread group not found, please call initializeQuery first.");
+    {
+        LOG_ERROR(getLogger("QueryContext"), "Thread group not found, please call initializeQuery first.");
+        return 0;
+    }
     return CurrentThread::getGroup()->memory_tracker.get();
 }
 
 double currentThreadGroupMemoryUsageRatio()
 {
     if (!CurrentThread::getGroup())
-        throw DB::Exception(ErrorCodes::LOGICAL_ERROR, "Thread group not found, please call initializeQuery first.");
+    {
+        LOG_ERROR(getLogger("QueryContext"), "Thread group not found, please call initializeQuery first.");
+        return 0;
+    }
     return static_cast<double>(CurrentThread::getGroup()->memory_tracker.get()) / CurrentThread::getGroup()->memory_tracker.getSoftLimit();
 }
 }
