@@ -762,7 +762,7 @@ private:
         bool document_ok = false;
         if (col_json_const)
         {
-            std::string_view json{reinterpret_cast<const char *>(chars.data()), offsets[0] - 1};
+            std::string_view json{reinterpret_cast<const char *>(chars.data()), offsets[0]};
             document_ok = safeParseJson(json, parser, document);
         }
 
@@ -778,7 +778,7 @@ private:
         {
             if (!col_json_const)
             {
-                std::string_view json{reinterpret_cast<const char *>(&chars[offsets[i - 1]]), offsets[i] - offsets[i - 1] - 1};
+                std::string_view json{reinterpret_cast<const char *>(&chars[offsets[i - 1]]), offsets[i] - offsets[i - 1]};
                 document_ok = safeParseJson(json, parser, document);
             }
             if (document_ok)
