@@ -290,7 +290,7 @@ MergeTreeData::LoadPartResult SparkStorageMergeTree::loadDataPart(
 
     // without it "test mergetree optimize partitioned by one low card column" will log ERROR
     resetColumnSizes();
-    calculateColumnAndSecondaryIndexSizesIfNeeded();
+    calculateColumnAndSecondaryIndexSizesImpl();
 
     LOG_TRACE(log, "Finished loading {} part {} on disk {}", magic_enum::enum_name(to_state), part_name, part_disk_ptr->getName());
     return res;
