@@ -16,6 +16,8 @@
  */
 #pragma once
 #include <Columns/ColumnDecimal.h>
+#include <Columns/ColumnString.h>
+#include <Columns/ColumnVector.h>
 #include <Core/Field.h>
 #include <base/Decimal_fwd.h>
 #include <parquet/schema.h>
