@@ -24,6 +24,7 @@ namespace {
 const std::string kHashShuffleName = "hash";
 const std::string kSortShuffleName = "sort";
 const std::string kRssSortShuffleName = "rss_sort";
+const std::string kGpuHashShuffleName = "gpu_hash";
 } // namespace
 
 ShuffleWriterType ShuffleWriter::stringToType(const std::string& typeString) {
@@ -36,6 +37,9 @@ ShuffleWriterType ShuffleWriter::stringToType(const std::string& typeString) {
   if (typeString == kRssSortShuffleName) {
     return ShuffleWriterType::kRssSortShuffle;
   }
+  if (typeString == kGpuHashShuffleName) {
+    return ShuffleWriterType::kGpuHashShuffle;
+  }
   throw GlutenException("Unrecognized shuffle writer type: " + typeString);
 }
 
@@ -47,6 +51,8 @@ std::string ShuffleWriter::typeToString(ShuffleWriterType type) {
       return kSortShuffleName;
     case ShuffleWriterType::kRssSortShuffle:
       return kRssSortShuffleName;
+    case ShuffleWriterType::kGpuHashShuffle:
+      return kGpuHashShuffleName;
   }
   GLUTEN_UNREACHABLE();
 }
@@ -55,8 +61,8 @@ int32_t ShuffleWriter::numPartitions() const {
   return numPartitions_;
 }
 
-int64_t ShuffleWriter::totalBytesWritten() const {
-  return metrics_.totalBytesWritten;
+int64_t ShuffleWriter::bytesWritten() const {
+  return writtenBytes_;
 }
 
 int64_t ShuffleWriter::totalBytesEvicted() const {
