@@ -23,11 +23,7 @@ namespace gluten {
 
 enum CodecBackend { NONE, QAT };
 
-// BUFFER mode will preallocate max compressed buffer, and then compress each buffer to the max compressed buffer
-// ROWVECTOR mode will copy the buffers to a big buffer and then compress the big buffer
-enum CompressionMode { BUFFER, ROWVECTOR };
-
-std::unique_ptr<arrow::util::Codec> createArrowIpcCodec(
+std::unique_ptr<arrow::util::Codec> createCompressionCodec(
     arrow::Compression::type compressedType,
     CodecBackend codecBackend,
     int32_t compressionLevel = arrow::util::kUseDefaultCompressionLevel);
