@@ -69,6 +69,7 @@ struct Metrics {
     kNumReplacedWithDynamicFilterRows,
     kFlushRowCount,
     kLoadedToValueHook,
+    kBloomFilterBlocksByteSize,
     kScanTime,
     kSkippedSplits,
     kProcessedSplits,
@@ -80,12 +81,18 @@ struct Metrics {
     kLocalReadBytes,
     kRamReadBytes,
     kPreloadSplits,
+    kPageLoadTime,
+    kDataSourceAddSplitWallNanos,
+    kDataSourceReadWallNanos,
 
     // Write metrics.
     kPhysicalWrittenBytes,
     kWriteIOTime,
     kNumWrittenFiles,
 
+    // Load lazy vector.
+    kLoadLazyVectorTime,
+
     // The end of enum items.
     kEnd,
     kNum = kEnd - kBegin
