@@ -35,10 +35,11 @@
 #include "shuffle/LocalPartitionWriter.h"
 #include "shuffle/VeloxShuffleWriter.h"
 #include "shuffle/rss/RssPartitionWriter.h"
+#include "tests/utils/LocalRssClient.h"
+#include "tests/utils/TestAllocationListener.h"
+#include "tests/utils/TestStreamReader.h"
 #include "utils/Exception.h"
-#include "utils/LocalRssClient.h"
 #include "utils/StringUtil.h"
-#include "utils/TestAllocationListener.h"
 #include "utils/Timer.h"
 #include "utils/VeloxArrowUtils.h"
 #include "velox/exec/PlanNodeStats.h"
@@ -185,7 +186,7 @@ std::unique_ptr<arrow::util::Codec> createCodec() {
 
   setCompressionTypeFromFlag(compressionType, codecBackend);
 
-  return createArrowIpcCodec(compressionType, codecBackend);
+  return createCompressionCodec(compressionType, codecBackend);
 }
 
 std::shared_ptr<PartitionWriter>
@@ -254,7 +255,7 @@ void populateWriterMetrics(
   }
   metrics.dataSize +=
       std::accumulate(shuffleWriter->rawPartitionLengths().begin(), shuffleWriter->rawPartitionLengths().end(), 0LL);
-  metrics.bytesWritten += shuffleWriter->totalBytesWritten();
+  metrics.bytesWritten += shuffleWriter->bytesWritten();
   metrics.bytesSpilled += shuffleWriter->totalBytesEvicted();
 }
 
@@ -304,8 +305,9 @@ void runShuffle(
     const auto reader = createShuffleReader(runtime, schema);
 
     GLUTEN_ASSIGN_OR_THROW(auto in, arrow::io::ReadableFile::Open(dataFile));
+    auto streamReader = std::make_shared<TestStreamReader>(std::move(in));
     // Read all partitions.
-    auto iter = reader->readStream(in);
+    auto iter = reader->read(streamReader);
     while (iter->hasNext()) {
       // Read and discard.
       auto cb = iter->next();
@@ -400,6 +402,7 @@ auto BM_Generic = [](::benchmark::State& state,
 
   auto plan = getPlanFromFile("Plan", planFile);
   std::vector<std::string> splits{};
+  splits.reserve(splitFiles.size());
   for (const auto& splitFile : splitFiles) {
     splits.push_back(getPlanFromFile("ReadRel.LocalFiles", splitFile));
   }
@@ -426,12 +429,12 @@ auto BM_Generic = [](::benchmark::State& state,
     ScopedTimer timer(&elapsedTime);
     for (auto _ : state) {
       std::vector<std::shared_ptr<gluten::ResultIterator>> inputIters;
+      inputIters.reserve(dataFiles.size());
       std::vector<FileReaderIterator*> inputItersRaw;
       if (!dataFiles.empty()) {
         for (const auto& input : dataFiles) {
-          inputIters.push_back(
-              FileReaderIterator::getInputIteratorFromFileReader(
-                  readerType, input, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool()));
+          inputIters.push_back(FileReaderIterator::getInputIteratorFromFileReader(
+              readerType, input, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool()));
         }
         std::transform(
             inputIters.begin(),
@@ -448,7 +451,8 @@ auto BM_Generic = [](::benchmark::State& state,
         runtime->parseSplitInfo(reinterpret_cast<uint8_t*>(split.data()), split.size(), i);
       }
 
-      auto resultIter = runtime->createResultIterator(veloxSpillDir, std::move(inputIters), runtime->getConfMap());
+      auto resultIter = runtime->createResultIterator(veloxSpillDir, std::move(inputIters));
+      runtime->noMoreSplits(resultIter.get());
       listenerPtr->setIterator(resultIter.get());
 
       if (FLAGS_with_shuffle) {
