@@ -17,6 +17,8 @@
 
 #include "VeloxRuntime.h"
 
+#include <operators/plannodes/RowVectorStream.h>
+
 #include <algorithm>
 #include <filesystem>
 
@@ -52,6 +54,10 @@ DECLARE_bool(velox_memory_pool_capacity_transfer_across_tasks);
 #include "operators/writer/VeloxParquetDataSourceABFS.h"
 #endif
 
+#ifdef GLUTEN_ENABLE_GPU
+#include "operators/serializer/VeloxGpuColumnarBatchSerializer.h"
+#endif
+
 using namespace facebook;
 
 namespace gluten {
@@ -88,7 +94,7 @@ void VeloxRuntime::parsePlan(const uint8_t* data, int32_t size) {
           << std::string(50, '#') << " received substrait::Plan: " << taskInfo_.value() << std::endl
           << planJson;
     } catch (const std::exception& e) {
-      LOG(WARNING) << "Error converting Substrait plan to JSON: " << e.what();
+      LOG(WARNING) << "Error converting substrait::Plan to JSON: " << e.what();
     }
   }
 
@@ -106,7 +112,7 @@ void VeloxRuntime::parseSplitInfo(const uint8_t* data, int32_t size, int32_t spl
           << std::string(50, '#') << " received substrait::ReadRel.LocalFiles: " << taskInfo_.value() << std::endl
           << splitJson;
     } catch (const std::exception& e) {
-      LOG(WARNING) << "Error converting Substrait plan to JSON: " << e.what();
+      LOG(WARNING) << "Error converting substrait::ReadRel.LocalFiles to JSON: " << e.what();
     }
   }
   ::substrait::ReadRel_LocalFiles localFile;
@@ -120,26 +126,34 @@ void VeloxRuntime::getInfoAndIds(
     std::vector<std::shared_ptr<SplitInfo>>& scanInfos,
     std::vector<velox::core::PlanNodeId>& scanIds,
     std::vector<velox::core::PlanNodeId>& streamIds) {
+  int32_t streamIdx = 0;
   for (const auto& leafPlanNodeId : leafPlanNodeIds) {
     auto it = splitInfoMap.find(leafPlanNodeId);
     if (it == splitInfoMap.end()) {
       throw std::runtime_error("Could not find leafPlanNodeId.");
     }
     auto splitInfo = it->second;
-    if (splitInfo->isStream) {
-      streamIds.emplace_back(leafPlanNodeId);
-    } else {
-      scanInfos.emplace_back(splitInfo);
+    // Based on the current code, indexing of streams and files follow different orders:
+    // 1. Streams follow "iterator:<idx>" in the substrait plan;
+    // 2. Files follow the traversal order in the plan node tree.
+    // FIXME: Why we didn't have a unified design?
+    switch (splitInfo->leafType) {
+    case SplitInfo::LeafType::SPLIT_AWARE_STREAM:
+      streamIds.emplace_back(ValueStreamConnectorFactory::nodeIdOf(streamIdx++));
+break;
+      case SplitInfo::LeafType::TABLE_SCAN:
+        scanInfos.emplace_back(splitInfo);
       scanIds.emplace_back(leafPlanNodeId);
+break;
+      case SplitInfo::LeafType::TRIVIAL_LEAF:
+break;
     }
   }
 }
 
 std::string VeloxRuntime::planString(bool details, const std::unordered_map<std::string, std::string>& sessionConf) {
-  std::vector<std::shared_ptr<ResultIterator>> inputs;
   auto veloxMemoryPool = gluten::defaultLeafVeloxMemoryPool();
-  VeloxPlanConverter veloxPlanConverter(
-      inputs, veloxMemoryPool.get(), veloxCfg_.get(), std::nullopt, std::nullopt, true);
+  VeloxPlanConverter veloxPlanConverter(veloxMemoryPool.get(), veloxCfg_.get(), {}, std::nullopt, std::nullopt, true);
   auto veloxPlan = veloxPlanConverter.toVeloxPlan(substraitPlan_, localFiles_);
   return veloxPlan->toString(details, true);
 }
@@ -152,14 +166,13 @@ VeloxMemoryManager* VeloxRuntime::memoryManager() {
 
 std::shared_ptr<ResultIterator> VeloxRuntime::createResultIterator(
     const std::string& spillDir,
-    const std::vector<std::shared_ptr<ResultIterator>>& inputs,
-    const std::unordered_map<std::string, std::string>& sessionConf) {
+    const std::vector<std::shared_ptr<ResultIterator>>& inputs) {
   LOG_IF(INFO, debugModeEnabled_) << "VeloxRuntime session config:" << printConfig(confMap_);
 
   VeloxPlanConverter veloxPlanConverter(
-      inputs,
       memoryManager()->getLeafMemoryPool().get(),
       veloxCfg_.get(),
+      inputs,
       *localWriteFilesTempPath(),
       *localWriteFileName());
   veloxPlan_ = veloxPlanConverter.toVeloxPlan(substraitPlan_, std::move(localFiles_));
@@ -182,11 +195,26 @@ std::shared_ptr<ResultIterator> VeloxRuntime::createResultIterator(
       scanInfos,
       streamIds,
       spillDir,
-      sessionConf,
+      veloxCfg_,
       taskInfo_.has_value() ? taskInfo_.value() : SparkTaskInfo{});
+
+  auto remainingInputIterators = veloxPlanConverter.remainingInputIterators();
+  if (!remainingInputIterators.empty()) {
+  // Converts remaining input iterators to splits and add them to the task.
+    wholeStageIter->addIteratorSplits(remainingInputIterators);
+  }
+
   return std::make_shared<ResultIterator>(std::move(wholeStageIter), this);
 }
 
+void VeloxRuntime::noMoreSplits(ResultIterator* iter){
+    auto* splitAwareIter = dynamic_cast<gluten::SplitAwareColumnarBatchIterator*>(iter->getInputIter());
+    if (splitAwareIter == nullptr) {
+      throw GlutenException("Iterator does not support split management");
+    }
+    splitAwareIter->noMoreSplits();
+}
+
 std::shared_ptr<ColumnarToRowConverter> VeloxRuntime::createColumnar2RowConverter(int64_t column2RowMemThreshold) {
   auto veloxPool = memoryManager()->getLeafMemoryPool();
   return std::make_shared<VeloxColumnarToRowConverter>(veloxPool, column2RowMemThreshold);
@@ -223,13 +251,16 @@ std::shared_ptr<IcebergWriter> VeloxRuntime::createIcebergWriter(
     int32_t format,
     const std::string& outputDirectory,
     facebook::velox::common::CompressionKind compressionKind,
+    int32_t partitionId,
+    int64_t taskId,
+    const std::string& operationId,
     std::shared_ptr<const facebook::velox::connector::hive::iceberg::IcebergPartitionSpec> spec,
     const gluten::IcebergNestedField& protoField,
     const std::unordered_map<std::string, std::string>& sparkConfs) {
   auto veloxPool = memoryManager()->getLeafMemoryPool();
   auto connectorPool = memoryManager()->getAggregateMemoryPool();
   return std::make_shared<IcebergWriter>(
-      rowType, format, outputDirectory, compressionKind, spec, protoField, sparkConfs, veloxPool, connectorPool);
+      rowType, format, outputDirectory, compressionKind, partitionId, taskId, operationId, spec, protoField, sparkConfs, veloxPool, connectorPool);
 }
 #endif
 
@@ -287,7 +318,7 @@ std::shared_ptr<VeloxDataSource> VeloxRuntime::createDataSource(
 std::shared_ptr<ShuffleReader> VeloxRuntime::createShuffleReader(
     std::shared_ptr<arrow::Schema> schema,
     ShuffleReaderOptions options) {
-  auto codec = gluten::createArrowIpcCodec(options.compressionType, options.codecBackend);
+  auto codec = gluten::createCompressionCodec(options.compressionType, options.codecBackend);
   const auto veloxCompressionKind = arrowCompressionTypeToVelox(options.compressionType);
   const auto rowType = facebook::velox::asRowType(gluten::fromArrowSchema(schema));
 
@@ -299,8 +330,7 @@ std::shared_ptr<ShuffleReader> VeloxRuntime::createShuffleReader(
       options.batchSize,
       options.readerBufferSize,
       options.deserializerBufferSize,
-      memoryManager()->defaultArrowMemoryPool(),
-      memoryManager()->getLeafMemoryPool(),
+      memoryManager(),
       options.shuffleWriterType);
 
   return std::make_shared<VeloxShuffleReader>(std::move(deserializerFactory));
@@ -309,6 +339,11 @@ std::shared_ptr<ShuffleReader> VeloxRuntime::createShuffleReader(
 std::unique_ptr<ColumnarBatchSerializer> VeloxRuntime::createColumnarBatchSerializer(struct ArrowSchema* cSchema) {
   auto arrowPool = memoryManager()->defaultArrowMemoryPool();
   auto veloxPool = memoryManager()->getLeafMemoryPool();
+#ifdef GLUTEN_ENABLE_GPU
+  if (veloxCfg_->get<bool>(kCudfEnabled, kCudfEnabledDefault)) {
+    return std::make_unique<VeloxGpuColumnarBatchSerializer>(arrowPool, veloxPool, cSchema);
+  }
+#endif
   return std::make_unique<VeloxColumnarBatchSerializer>(arrowPool, veloxPool, cSchema);
 }
 
