@@ -41,6 +41,7 @@
 
 #ifdef GLUTEN_ENABLE_GPU
 #include "cudf/CudfPlanValidator.h"
+#include "utils/GpuBufferBatchResizer.h"
 #endif
 
 #ifdef GLUTEN_ENABLE_ENHANCED_FEATURES
@@ -56,6 +57,9 @@ jmethodID infoClsInitMethod;
 
 jclass blockStripesClass;
 jmethodID blockStripesConstructor;
+
+jclass batchWriteMetricsClass;
+jmethodID batchWriteMetricsConstructor;
 } // namespace
 
 #ifdef __cplusplus
@@ -80,6 +84,10 @@ jint JNI_OnLoad(JavaVM* vm, void*) {
       createGlobalClassReferenceOrError(env, "Lorg/apache/spark/sql/execution/datasources/BlockStripes;");
   blockStripesConstructor = getMethodIdOrError(env, blockStripesClass, "<init>", "(J[J[II[[B)V");
 
+  batchWriteMetricsClass =
+    createGlobalClassReferenceOrError(env, "Lorg/apache/gluten/metrics/BatchWriteMetrics;");
+  batchWriteMetricsConstructor = getMethodIdOrError(env, batchWriteMetricsClass, "<init>", "(JIJJ)V");
+
   DLOG(INFO) << "Loaded Velox backend.";
 
   return jniVersion;
@@ -175,6 +183,60 @@ Java_org_apache_gluten_vectorized_PlanEvaluatorJniWrapper_nativeValidateWithFail
   JNI_METHOD_END(nullptr)
 }
 
+JNIEXPORT jboolean JNICALL
+Java_org_apache_gluten_vectorized_PlanEvaluatorJniWrapper_nativeValidateExpression( // NOLINT
+    JNIEnv* env,
+    jobject wrapper,
+    jbyteArray exprArray,
+    jbyteArray inputTypeArray,
+    jobjectArray mappings) {
+  JNI_METHOD_START
+  auto safeExprArray = getByteArrayElementsSafe(env, exprArray);
+  auto safeInputTypeArray = getByteArrayElementsSafe(env, inputTypeArray);
+  auto exprData = safeExprArray.elems();
+  auto exprSize = env->GetArrayLength(exprArray);
+  auto inputTypeData = safeInputTypeArray.elems();
+  auto inputTypeSize = env->GetArrayLength(inputTypeArray);
+
+  ::substrait::Expression expression;
+  parseProtobuf(exprData, exprSize, &expression);
+  ::substrait::Type inputSubstraitType;
+  parseProtobuf(inputTypeData, inputTypeSize, &inputSubstraitType);
+
+  // Get the function mappings.
+  auto mappingSize = env->GetArrayLength(mappings);
+  std::unordered_map<uint64_t, std::string> functionMappings;
+  for (jsize i = 0; i < mappingSize; ++i) {
+    jbyteArray mapping = (jbyteArray)env->GetObjectArrayElement(mappings, i);
+    auto safeMappingArray = getByteArrayElementsSafe(env, mapping);
+    auto mappingData = safeMappingArray.elems();
+    auto mappingSize = env->GetArrayLength(mapping);
+
+    ::substrait::extensions::SimpleExtensionDeclaration mappingDecl;
+    parseProtobuf(mappingData, mappingSize, &mappingDecl);
+
+    const auto& sFmap = mappingDecl.extension_function();
+    auto id = sFmap.function_anchor();
+    auto name = sFmap.name();
+    functionMappings.emplace(id, name);
+    env->DeleteLocalRef(mapping);
+  }
+
+  auto pool = defaultLeafVeloxMemoryPool().get();
+  SubstraitToVeloxPlanValidator planValidator(pool);
+  auto inputType = SubstraitParser::parseType(inputSubstraitType);
+  if (inputType->kind() != TypeKind::ROW) {
+    throw GlutenException("Input type is not a RowType.");
+  }
+  auto rowType = std::dynamic_pointer_cast<const RowType>(inputType);
+  try {
+    return planValidator.validate(expression, rowType, std::move(functionMappings));
+  } catch (std::invalid_argument& e) {
+    return false;
+  }
+  JNI_METHOD_END(false)
+}
+
 JNIEXPORT jlong JNICALL Java_org_apache_gluten_columnarbatch_VeloxColumnarBatchJniWrapper_from( // NOLINT
     JNIEnv* env,
     jobject wrapper,
@@ -201,6 +263,7 @@ JNIEXPORT jlong JNICALL Java_org_apache_gluten_columnarbatch_VeloxColumnarBatchJ
   auto safeArray = getLongArrayElementsSafe(env, batchHandles);
 
   std::vector<std::shared_ptr<ColumnarBatch>> batches;
+  batches.reserve(handleCount);
   for (int i = 0; i < handleCount; ++i) {
     int64_t handle = safeArray.elems()[i];
     auto batch = ObjectStore::retrieve<ColumnarBatch>(handle);
@@ -211,6 +274,55 @@ JNIEXPORT jlong JNICALL Java_org_apache_gluten_columnarbatch_VeloxColumnarBatchJ
   JNI_METHOD_END(kInvalidObjectHandle)
 }
 
+JNIEXPORT jlong JNICALL Java_org_apache_gluten_columnarbatch_VeloxColumnarBatchJniWrapper_repeatedThenCompose( // NOLINT
+    JNIEnv* env,
+    jobject wrapper,
+    jlong repeatedBatchHandle,
+    jlong nonRepeatedBatchHandle,
+    jintArray rowId2RowNums) {
+  JNI_METHOD_START
+  auto ctx = getRuntime(env, wrapper);
+  auto runtime = dynamic_cast<VeloxRuntime*>(ctx);
+
+  int rowId2RowNumsSize = env->GetArrayLength(rowId2RowNums);
+  auto safeRowId2RowNumsArray = getIntArrayElementsSafe(env, rowId2RowNums);
+
+  auto veloxPool = runtime->memoryManager()->getLeafMemoryPool();
+  vector_size_t rowNums = 0;
+  for (int i = 0; i < rowId2RowNumsSize; ++i) {
+    rowNums += safeRowId2RowNumsArray.elems()[i];
+  }
+
+  // Create a indices vector.
+  // The indices will be used to create a dictionary vector for the first batch.
+  auto repeatedIndices = AlignedBuffer::allocate<vector_size_t>(rowNums, veloxPool.get(), 0);
+  auto* rawRepeatedIndices = repeatedIndices->asMutable<vector_size_t>();
+  int lastRowIndexEnd = 0;
+  for (int i = 0; i < rowId2RowNumsSize; ++i) {
+    auto rowNum = safeRowId2RowNumsArray.elems()[i];
+    std::fill(rawRepeatedIndices + lastRowIndexEnd, rawRepeatedIndices + lastRowIndexEnd + rowNum, i);
+    lastRowIndexEnd += rowNum;
+  }
+
+  auto repeatedBatch = ObjectStore::retrieve<ColumnarBatch>(repeatedBatchHandle);
+  auto nonRepeatedBatch = ObjectStore::retrieve<ColumnarBatch>(nonRepeatedBatchHandle);
+  GLUTEN_CHECK(rowNums == nonRepeatedBatch->numRows(), "Row numbers after repeated do not match the expected size");
+
+  // wrap repeatedBatch's rowVector in dictionary vector.
+  auto vb = std::dynamic_pointer_cast<VeloxColumnarBatch>(repeatedBatch);
+  auto rowVector = vb->getRowVector();
+  std::vector<VectorPtr> outputs(rowVector->childrenSize());
+  for (int i = 0; i < outputs.size(); i++) {
+    outputs[i] = BaseVector::wrapInDictionary(nullptr /*nulls*/, repeatedIndices, rowNums, rowVector->childAt(i));
+  }
+  auto newRowVector =
+      std::make_shared<RowVector>(veloxPool.get(), rowVector->type(), BufferPtr(nullptr), rowNums, std::move(outputs));
+  repeatedBatch = std::make_shared<VeloxColumnarBatch>(std::move(newRowVector));
+  auto newBatch = VeloxColumnarBatch::compose(veloxPool.get(), {std::move(repeatedBatch), std::move(nonRepeatedBatch)});
+  return ctx->saveObject(newBatch);
+  JNI_METHOD_END(kInvalidObjectHandle)
+}
+
 JNIEXPORT jlong JNICALL Java_org_apache_gluten_utils_VeloxBloomFilterJniWrapper_empty( // NOLINT
     JNIEnv* env,
     jobject wrapper,
@@ -321,17 +433,36 @@ JNIEXPORT jlong JNICALL Java_org_apache_gluten_utils_VeloxBatchResizerJniWrapper
     jobject wrapper,
     jint minOutputBatchSize,
     jint maxOutputBatchSize,
+    jlong preferredBatchBytes,
     jobject jIter) {
   JNI_METHOD_START
   auto ctx = getRuntime(env, wrapper);
   auto pool = dynamic_cast<VeloxMemoryManager*>(ctx->memoryManager())->getLeafMemoryPool();
   auto iter = makeJniColumnarBatchIterator(env, jIter, ctx);
   auto appender = std::make_shared<ResultIterator>(
-      std::make_unique<VeloxBatchResizer>(pool.get(), minOutputBatchSize, maxOutputBatchSize, std::move(iter)));
+      std::make_unique<VeloxBatchResizer>(pool.get(), minOutputBatchSize, maxOutputBatchSize, preferredBatchBytes, std::move(iter)));
   return ctx->saveObject(appender);
   JNI_METHOD_END(kInvalidObjectHandle)
 }
 
+#ifdef GLUTEN_ENABLE_GPU
+JNIEXPORT jlong JNICALL Java_org_apache_gluten_utils_GpuBufferBatchResizerJniWrapper_create( // NOLINT
+    JNIEnv* env,
+    jobject wrapper,
+    jint minOutputBatchSize,
+    jobject jIter) {
+  JNI_METHOD_START
+  auto ctx = getRuntime(env, wrapper);
+  auto arrowPool = dynamic_cast<VeloxMemoryManager*>(ctx->memoryManager())->defaultArrowMemoryPool();
+  auto pool = dynamic_cast<VeloxMemoryManager*>(ctx->memoryManager())->getLeafMemoryPool();
+  auto iter = makeJniColumnarBatchIterator(env, jIter, ctx);
+  auto appender = std::make_shared<ResultIterator>(
+      std::make_unique<GpuBufferBatchResizer>(arrowPool, pool.get(), minOutputBatchSize, std::move(iter)));
+  return ctx->saveObject(appender);
+  JNI_METHOD_END(kInvalidObjectHandle)
+}
+#endif
+
 JNIEXPORT jboolean JNICALL
 Java_org_apache_gluten_utils_VeloxFileSystemValidationJniWrapper_allSupportedByRegisteredFileSystems( // NOLINT
     JNIEnv* env,
@@ -345,6 +476,7 @@ Java_org_apache_gluten_utils_VeloxFileSystemValidationJniWrapper_allSupportedByR
     if (!velox::filesystems::isPathSupportedByRegisteredFileSystems(path)) {
       return false;
     }
+    env->DeleteLocalRef(string);
   }
   return true;
   JNI_METHOD_END(false)
@@ -450,8 +582,13 @@ Java_org_apache_gluten_datasource_VeloxDataSourceJniWrapper_splitBlockByPartitio
   const auto inputRowVector = veloxBatch->getRowVector();
   const auto numRows = inputRowVector->size();
 
-  connector::hive::PartitionIdGenerator idGen{
-      asRowType(inputRowVector->type()), partitionColIndicesVec, 128, pool.get(), true};
+  connector::hive::PartitionIdGenerator idGen(
+      asRowType(inputRowVector->type()), partitionColIndicesVec, 65536, pool.get()
+#ifdef GLUTEN_ENABLE_ENHANCED_FEATURES
+      ,
+      true
+#endif    
+    );
   raw_vector<uint64_t> partitionIds{};
   idGen.run(inputRowVector, partitionIds);
   GLUTEN_CHECK(partitionIds.size() == numRows, "Mismatched number of partition ids");
@@ -559,7 +696,7 @@ JNIEXPORT void JNICALL Java_org_apache_gluten_monitor_VeloxMemoryProfiler_start(
   JNI_METHOD_START
 #ifdef ENABLE_JEMALLOC_STATS
   bool active = true;
-  mallctl("prof.active", NULL, NULL, &active, sizeof(bool));
+  mallctl("prof.active", nullptr, nullptr, &active, sizeof(bool));
 #endif
   JNI_METHOD_END()
 }
@@ -569,7 +706,7 @@ JNIEXPORT void JNICALL Java_org_apache_gluten_monitor_VeloxMemoryProfiler_dump(
     jclass) {
   JNI_METHOD_START
 #ifdef ENABLE_JEMALLOC_STATS
-  mallctl("prof.dump", NULL, NULL, NULL, 0);
+  mallctl("prof.dump", nullptr, nullptr, nullptr, 0);
 #endif
   JNI_METHOD_END()
 }
@@ -580,7 +717,7 @@ JNIEXPORT void JNICALL Java_org_apache_gluten_monitor_VeloxMemoryProfiler_stop(
   JNI_METHOD_START
 #ifdef ENABLE_JEMALLOC_STATS
   bool active = false;
-  mallctl("prof.active", NULL, NULL, &active, sizeof(bool));
+  mallctl("prof.active", nullptr, nullptr, &active, sizeof(bool));
 #endif
   JNI_METHOD_END()
 }
@@ -619,7 +756,8 @@ Java_org_apache_gluten_vectorized_CelebornPartitionWriterJniWrapper_createPartit
 
   auto partitionWriter = std::make_shared<RssPartitionWriter>(
       numPartitions,
-      createArrowIpcCodec(getCompressionType(env, codecJstr), getCodecBackend(env, codecBackendJstr), compressionLevel),
+      createCompressionCodec(
+          getCompressionType(env, codecJstr), getCodecBackend(env, codecBackendJstr), compressionLevel),
       ctx->memoryManager(),
       partitionWriterOptions,
       celebornClient);
@@ -662,7 +800,8 @@ Java_org_apache_gluten_vectorized_UnifflePartitionWriterJniWrapper_createPartiti
 
   auto partitionWriter = std::make_shared<RssPartitionWriter>(
       numPartitions,
-      createArrowIpcCodec(getCompressionType(env, codecJstr), getCodecBackend(env, codecBackendJstr), compressionLevel),
+      createCompressionCodec(
+          getCompressionType(env, codecJstr), getCodecBackend(env, codecBackendJstr), compressionLevel),
       ctx->memoryManager(),
       partitionWriterOptions,
       uniffleClient);
@@ -705,8 +844,11 @@ JNIEXPORT jlong JNICALL Java_org_apache_gluten_execution_IcebergWriteJniWrapper_
     jint format,
     jstring directory,
     jstring codecJstr,
+    jint partitionId,
+    jlong taskId,
+    jstring operationId,
     jbyteArray partition,
-  jbyteArray fieldBytes) {
+    jbyteArray fieldBytes) {
   JNI_METHOD_START
   auto ctx = getRuntime(env, wrapper);
   auto runtime = dynamic_cast<VeloxRuntime*>(ctx);
@@ -726,6 +868,9 @@ JNIEXPORT jlong JNICALL Java_org_apache_gluten_execution_IcebergWriteJniWrapper_
       format,
       jStringToCString(env, directory),
       facebook::velox::common::stringToCompressionKind(jStringToCString(env, codecJstr)),
+      partitionId,
+      taskId,
+      jStringToCString(env, operationId),
       spec,
       protoField,
       sparkConf));
@@ -760,6 +905,25 @@ JNIEXPORT jobjectArray JNICALL Java_org_apache_gluten_execution_IcebergWriteJniW
 
   JNI_METHOD_END(nullptr)
 }
+
+JNIEXPORT jobject JNICALL Java_org_apache_gluten_execution_IcebergWriteJniWrapper_metrics( // NOLINT
+    JNIEnv* env,
+    jobject wrapper,
+    jlong writerHandle) {
+  JNI_METHOD_START
+  auto writer = ObjectStore::retrieve<IcebergWriter>(writerHandle);
+  auto writeStats = writer->writeStats();
+  jobject writeMetrics = env->NewObject(
+    batchWriteMetricsClass,
+    batchWriteMetricsConstructor,
+    writeStats.numWrittenBytes,
+    writeStats.numWrittenFiles,
+    writeStats.writeIOTimeNs,
+    writeStats.writeWallNs);
+  return writeMetrics;
+
+  JNI_METHOD_END(nullptr)
+}
 #endif
 
 #ifdef __cplusplus
