@@ -131,7 +131,7 @@ TypePtr getScalarType(const ::substrait::Expression::Literal& literal) {
     }
     case ::substrait::Expression_Literal::LiteralTypeCase::kDate:
       return DATE();
-    case ::substrait::Expression_Literal::LiteralTypeCase::kTimestamp:
+    case ::substrait::Expression_Literal::LiteralTypeCase::kTimestampTz:
       return TIMESTAMP();
     case ::substrait::Expression_Literal::LiteralTypeCase::kString:
       return VARCHAR();
@@ -179,13 +179,13 @@ std::shared_ptr<core::ConstantTypedExpr> constructConstantVector(
     const TypePtr& type) {
   VELOX_CHECK(type->isPrimitiveType());
   if (substraitLit.has_binary()) {
-    return std::make_shared<core::ConstantTypedExpr>(
-        type, variant::binary(gluten::SubstraitParser::getLiteralValue<StringView>(substraitLit)));
-  } else {
-    using T = typename TypeTraits<kind>::NativeType;
-    return std::make_shared<core::ConstantTypedExpr>(
-        type, variant(gluten::SubstraitParser::getLiteralValue<T>(substraitLit)));
+    const auto sv = gluten::SubstraitParser::getLiteralValue<StringView>(substraitLit);
+    return std::make_shared<core::ConstantTypedExpr>(type, variant::binary(sv.str()));
   }
+
+  using T = typename TypeTraits<kind>::NativeType;
+  return std::make_shared<core::ConstantTypedExpr>(
+      type, variant(gluten::SubstraitParser::getLiteralValue<T>(substraitLit)));
 }
 
 core::FieldAccessTypedExprPtr
