@@ -31,9 +31,29 @@
 #include "config/GlutenConfig.h"
 #include "config/VeloxConfig.h"
 
+#ifdef GLUTEN_ENABLE_GPU
+#include "operators/plannodes/CudfVectorStream.h"
+#include "velox/experimental/cudf/connectors/hive/CudfHiveDataSink.h"
+#include "velox/experimental/cudf/connectors/hive/CudfHiveTableHandle.h"
+#include "velox/experimental/cudf/exec/ToCudf.h"
+#include "velox/experimental/cudf/exec/VeloxCudfInterop.h"
+using namespace cudf_velox::connector::hive;
+#endif
+
 namespace gluten {
 namespace {
 
+bool useCudfTableHandle(const std::vector<std::shared_ptr<SplitInfo>>& splitInfos) {
+#ifdef GLUTEN_ENABLE_GPU
+  if (splitInfos.empty()) {
+    return false;
+  }
+  return splitInfos[0]->canUseCudfConnector();
+#else
+  return false;
+#endif
+}
+
 core::SortOrder toSortOrder(const ::substrait::SortField& sortField) {
   switch (sortField.direction()) {
     case ::substrait::SortField_SortDirection_SORT_DIRECTION_ASC_NULLS_FIRST:
@@ -82,7 +102,7 @@ EmitInfo getEmitInfo(const ::substrait::RelCommon& relCommon, const core::PlanNo
 RowTypePtr getJoinInputType(const core::PlanNodePtr& leftNode, const core::PlanNodePtr& rightNode) {
   auto outputSize = leftNode->outputType()->size() + rightNode->outputType()->size();
   std::vector<std::string> outputNames;
-  std::vector<std::shared_ptr<const Type>> outputTypes;
+  std::vector<TypePtr> outputTypes;
   outputNames.reserve(outputSize);
   outputTypes.reserve(outputSize);
   for (const auto& node : {leftNode, rightNode}) {
@@ -119,7 +139,7 @@ RowTypePtr getJoinOutputType(
   if (outputMayIncludeLeftColumns) {
     if (core::isLeftSemiProjectJoin(joinType)) {
       std::vector<std::string> outputNames = leftNode->outputType()->names();
-      std::vector<std::shared_ptr<const Type>> outputTypes = leftNode->outputType()->children();
+      std::vector<TypePtr> outputTypes = leftNode->outputType()->children();
       outputNames.emplace_back("exists");
       outputTypes.emplace_back(BOOLEAN());
       return std::make_shared<const RowType>(std::move(outputNames), std::move(outputTypes));
@@ -131,7 +151,7 @@ RowTypePtr getJoinOutputType(
   if (outputMayIncludeRightColumns) {
     if (core::isRightSemiProjectJoin(joinType)) {
       std::vector<std::string> outputNames = rightNode->outputType()->names();
-      std::vector<std::shared_ptr<const Type>> outputTypes = rightNode->outputType()->children();
+      std::vector<TypePtr> outputTypes = rightNode->outputType()->children();
       outputNames.emplace_back("exists");
       outputTypes.emplace_back(BOOLEAN());
       return std::make_shared<const RowType>(std::move(outputNames), std::move(outputTypes));
@@ -142,8 +162,61 @@ RowTypePtr getJoinOutputType(
   VELOX_FAIL("Output should include left or right columns.");
 }
 
+// Get the function name suffix used by merge_extract companion function when having the same intermediate type across
+// signatures. Correponds to Velox 'toSuffixString', and the base name can be referred from
+// 'velox/expression/FunctionSignature.cpp'.
+std::string companionFunctionSuffix(const TypePtr& type) {
+  // For primitive and decimal types, return their names.
+  if (type->isDecimal()) {
+    return "DECIMAL";
+  }
+
+  if (type->isPrimitiveType()) {
+    return type->toString();
+  }
+
+  if (type->kind() == TypeKind::ARRAY) {
+    return "array_" + companionFunctionSuffix(std::dynamic_pointer_cast<const ArrayType>(type)->elementType());
+  }
+  if (type->kind() == TypeKind::MAP) {
+    auto mapType = std::dynamic_pointer_cast<const MapType>(type);
+    return "map_" + companionFunctionSuffix(mapType->keyType()) + "_" + companionFunctionSuffix(mapType->valueType());
+  }
+
+  std::string name;
+  if (type->kind() == TypeKind::ROW) {
+    name = "row";
+  }
+  std::string result = name;
+  const auto rowType = asRowType(type);
+  for (const auto& child : rowType->children()) {
+    result += '_';
+    result += companionFunctionSuffix(child);
+  }
+  result += "_end";
+  result += name;
+  return result;
+}
+
 } // namespace
 
+bool SplitInfo::canUseCudfConnector() {
+  bool isEmpty = partitionColumns.empty();
+
+  if (!isEmpty) {
+    // Check if all maps are empty
+    bool allMapsEmpty = true;
+    for (const auto& m : partitionColumns) {
+      if (!m.empty()) {
+        allMapsEmpty = false;
+        break;
+      }
+    }
+    isEmpty = allMapsEmpty;
+  }
+  return isEmpty && format == dwio::common::FileFormat::PARQUET;
+}
+
 core::PlanNodePtr SubstraitToVeloxPlanConverter::processEmit(
     const ::substrait::RelCommon& relCommon,
     const core::PlanNodePtr& noEmitNode) {
@@ -194,15 +267,29 @@ core::AggregationNode::Step SubstraitToVeloxPlanConverter::toAggregationFunction
 
 std::string SubstraitToVeloxPlanConverter::toAggregationFunctionName(
     const std::string& baseName,
-    const core::AggregationNode::Step& step) {
+    const core::AggregationNode::Step& step,
+    const TypePtr& resultType) {
   std::string suffix;
   switch (step) {
     case core::AggregationNode::Step::kPartial:
       suffix = "_partial";
       break;
-    case core::AggregationNode::Step::kFinal:
-      suffix = "_merge_extract";
-      break;
+    case core::AggregationNode::Step::kFinal: {
+      auto functionName = baseName + "_merge_extract";
+      auto signatures = exec::getAggregateFunctionSignatures(functionName);
+      if (signatures.has_value() && signatures.value().size() > 0) {
+        // The merge_extract function is registered without suffix.
+        return functionName;
+      }
+      // The merge_extract function must be registered with suffix based on result type.
+      functionName += ("_" + companionFunctionSuffix(resultType));
+      signatures = exec::getAggregateFunctionSignatures(functionName);
+      VELOX_CHECK(
+          signatures.has_value() && signatures.value().size() > 0,
+          "Cannot find function signature for {} in final aggregation step.",
+          functionName);
+      return functionName;
+    }
     case core::AggregationNode::Step::kIntermediate:
       suffix = "_merge";
       break;
@@ -350,9 +437,6 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
         VELOX_NYI("Unsupported Join type: {}", std::to_string(crossRel.type()));
       }
       break;
-    case ::substrait::CrossRel_JoinType::CrossRel_JoinType_JOIN_TYPE_OUTER:
-      joinType = core::JoinType::kFull;
-      break;
     default:
       VELOX_NYI("Unsupported Join type: {}", std::to_string(crossRel.type()));
   }
@@ -379,8 +463,10 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
   std::vector<core::FieldAccessTypedExprPtr> veloxGroupingExprs;
 
   // Get the grouping expressions.
-  for (const auto& grouping : aggRel.groupings()) {
-    for (const auto& groupingExpr : grouping.grouping_expressions()) {
+  VELOX_CHECK(
+      aggRel.groupings().size() <= 1, "At most one grouping is supported, but got {}.", aggRel.groupings().size());
+  if (aggRel.groupings().size() == 1) {
+    for (const auto& groupingExpr : aggRel.groupings()[0].grouping_expressions()) {
       // Velox's groupings are limited to be Field.
       veloxGroupingExprs.emplace_back(exprConverter_->toVeloxExpr(groupingExpr.selection(), inputType));
     }
@@ -402,22 +488,22 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
       }
     }
     const auto& aggFunction = measure.measure();
-    auto baseFuncName = SubstraitParser::findVeloxFunction(functionMap_, aggFunction.function_reference());
-    auto funcName = toAggregationFunctionName(baseFuncName, toAggregationFunctionStep(aggFunction));
     std::vector<core::TypedExprPtr> aggParams;
     aggParams.reserve(aggFunction.arguments().size());
     for (const auto& arg : aggFunction.arguments()) {
       aggParams.emplace_back(exprConverter_->toVeloxExpr(arg.value(), inputType));
     }
-    auto aggVeloxType = SubstraitParser::parseType(aggFunction.output_type());
-    auto aggExpr = std::make_shared<const core::CallTypedExpr>(aggVeloxType, std::move(aggParams), funcName);
 
+    auto aggVeloxType = SubstraitParser::parseType(aggFunction.output_type());
+    auto baseFuncName = SubstraitParser::findVeloxFunction(functionMap_, aggFunction.function_reference());
+    auto funcName = toAggregationFunctionName(baseFuncName, toAggregationFunctionStep(aggFunction), aggVeloxType);
+
+    auto aggExpr = std::make_shared<const core::CallTypedExpr>(aggVeloxType, std::move(aggParams), funcName);
     std::vector<TypePtr> rawInputTypes =
         SubstraitParser::sigToTypes(SubstraitParser::findFunctionSpec(functionMap_, aggFunction.function_reference()));
     aggregates.emplace_back(core::AggregationNode::Aggregate{aggExpr, rawInputTypes, mask, {}, {}});
   }
 
-  bool ignoreNullKeys = false;
   std::vector<core::FieldAccessTypedExprPtr> preGroupingExprs;
   if (aggRel.has_advanced_extension() &&
       SubstraitParser::configSetInOptimization(aggRel.advanced_extension(), "isStreaming=")) {
@@ -425,11 +511,6 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
     preGroupingExprs.insert(preGroupingExprs.begin(), veloxGroupingExprs.begin(), veloxGroupingExprs.end());
   }
 
-  if (aggRel.has_advanced_extension() &&
-      SubstraitParser::configSetInOptimization(aggRel.advanced_extension(), "ignoreNullKeys=")) {
-    ignoreNullKeys = true;
-  }
-
   // Get the output names of Aggregation.
   std::vector<std::string> aggOutNames;
   aggOutNames.reserve(aggRel.measures().size());
@@ -444,7 +525,8 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
       preGroupingExprs,
       aggOutNames,
       aggregates,
-      ignoreNullKeys,
+      /*ignoreNullKeys=*/false,
+      /*noGroupsSpanBatches=*/false,
       childNode);
 
   if (aggRel.has_common()) {
@@ -458,13 +540,13 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
   auto childNode = convertSingleInput<::substrait::ProjectRel>(projectRel);
   // Construct Velox Expressions.
   const auto& projectExprs = projectRel.expressions();
+  const auto& inputType = childNode->outputType();
+  const size_t totalSize = projectExprs.size() + inputType->size();
   std::vector<std::string> projectNames;
   std::vector<core::TypedExprPtr> expressions;
-  projectNames.reserve(projectExprs.size());
-  expressions.reserve(projectExprs.size());
+  projectNames.reserve(totalSize);
+  expressions.reserve(totalSize);
 
-  const auto& inputType = childNode->outputType();
-  int colIdx = 0;
   // Note that Substrait projection adds the project expressions on top of the
   // input to the projection node. Thus we need to add the input columns first
   // and then add the projection expressions.
@@ -475,27 +557,30 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
     const auto& fieldName = inputType->nameOf(idx);
     projectNames.emplace_back(fieldName);
     expressions.emplace_back(std::make_shared<core::FieldAccessTypedExpr>(inputType->childAt(idx), fieldName));
-    colIdx += 1;
   }
 
   // Then, adding project expression related project names and expressions.
-  for (const auto& expr : projectExprs) {
-    expressions.emplace_back(exprConverter_->toVeloxExpr(expr, inputType));
-    projectNames.emplace_back(SubstraitParser::makeNodeName(planNodeId_, colIdx));
-    colIdx += 1;
+  const size_t startIdx = expressions.size();
+  for (int i = 0; i < projectExprs.size(); i++) {
+    expressions.emplace_back(exprConverter_->toVeloxExpr(projectExprs[i], inputType));
+    projectNames.emplace_back(SubstraitParser::makeNodeName(planNodeId_, startIdx + i));
   }
 
   if (projectRel.has_common()) {
     auto relCommon = projectRel.common();
     const auto& emit = relCommon.emit();
     int emitSize = emit.output_mapping_size();
-    std::vector<std::string> emitProjectNames(emitSize);
-    std::vector<core::TypedExprPtr> emitExpressions(emitSize);
+    std::vector<std::string> emitProjectNames;
+    std::vector<core::TypedExprPtr> emitExpressions;
+    emitProjectNames.reserve(emitSize);
+    emitExpressions.reserve(emitSize);
+
     for (int i = 0; i < emitSize; i++) {
       int32_t mapId = emit.output_mapping(i);
-      emitProjectNames[i] = projectNames[mapId];
-      emitExpressions[i] = expressions[mapId];
+      emitProjectNames.emplace_back(std::move(projectNames[mapId]));
+      emitExpressions.emplace_back(std::move(expressions[mapId]));
     }
+
     return std::make_shared<core::ProjectNode>(
         nextPlanNodeId(), std::move(emitProjectNames), std::move(emitExpressions), std::move(childNode));
   } else {
@@ -582,6 +667,29 @@ std::shared_ptr<connector::hive::HiveInsertTableHandle> makeHiveInsertTableHandl
       writerOptions);
 }
 
+#ifdef GLUTEN_ENABLE_GPU
+std::shared_ptr<CudfHiveInsertTableHandle> makeCudfHiveInsertTableHandle(
+    const std::vector<std::string>& tableColumnNames,
+    const std::vector<TypePtr>& tableColumnTypes,
+    std::shared_ptr<cudf_velox::connector::hive::LocationHandle> locationHandle,
+    const std::optional<common::CompressionKind> compressionKind,
+    const std::unordered_map<std::string, std::string>& serdeParameters,
+    const std::shared_ptr<dwio::common::WriterOptions>& writerOptions) {
+  std::vector<std::shared_ptr<const CudfHiveColumnHandle>> columnHandles;
+  columnHandles.reserve(tableColumnNames.size());
+
+  for (int i = 0; i < tableColumnNames.size(); ++i) {
+    columnHandles.push_back(std::make_shared<CudfHiveColumnHandle>(
+        tableColumnNames.at(i),
+        tableColumnTypes.at(i),
+        cudf::data_type{cudf_velox::veloxToCudfTypeId(tableColumnTypes.at(i))}));
+  }
+
+  return std::make_shared<CudfHiveInsertTableHandle>(
+      columnHandles, locationHandle, compressionKind, serdeParameters, writerOptions);
+}
+#endif
+
 core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::WriteRel& writeRel) {
   core::PlanNodePtr childNode;
   if (writeRel.has_input()) {
@@ -675,25 +783,25 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
   // Spark's default compression code is snappy.
   const auto& compressionKind =
       writerOptions->compressionKind.value_or(common::CompressionKind::CompressionKind_SNAPPY);
-
+  std::shared_ptr<core::InsertTableHandle> tableHandle = std::make_shared<core::InsertTableHandle>(
+      kHiveConnectorId,
+      makeHiveInsertTableHandle(
+          tableColumnNames, /*inputType->names() clolumn name is different*/
+          inputType->children(),
+          partitionedKey,
+          bucketProperty,
+          makeLocationHandle(writePath, fileName, fileFormat, compressionKind, bucketProperty != nullptr),
+          writerOptions,
+          fileFormat,
+          compressionKind));
   return std::make_shared<core::TableWriteNode>(
       nextPlanNodeId(),
       inputType,
       tableColumnNames,
-      nullptr, /*aggregationNode*/
-      std::make_shared<core::InsertTableHandle>(
-          kHiveConnectorId,
-          makeHiveInsertTableHandle(
-              tableColumnNames, /*inputType->names() clolumn name is different*/
-              inputType->children(),
-              partitionedKey,
-              bucketProperty,
-              makeLocationHandle(writePath, fileName, fileFormat, compressionKind, bucketProperty != nullptr),
-              writerOptions,
-              fileFormat,
-              compressionKind)),
+      std::nullopt, /*columnStatsSpec*/
+      tableHandle,
       (!partitionedKey.empty()),
-      exec::TableWriteTraits::outputType(nullptr),
+      exec::TableWriteTraits::outputType(std::nullopt),
       connector::CommitStrategy::kNoCommit,
       childNode);
 }
@@ -791,12 +899,15 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
       SubstraitParser::configSetInOptimization(generateRel.advanced_extension(), "injectedProject=");
 
   if (injectedProject) {
-    // Child should be either ProjectNode or ValueStreamNode in case of project fallback.
+    // Child should be either ProjectNode or CudfValueStreamNode (GPU) in case of project fallback.
     VELOX_CHECK(
         (std::dynamic_pointer_cast<const core::ProjectNode>(childNode) != nullptr ||
-         std::dynamic_pointer_cast<const ValueStreamNode>(childNode) != nullptr) &&
-            childNode->outputType()->size() > requiredChildOutput.size(),
-        "injectedProject is true, but the ProjectNode or ValueStreamNode (in case of projection fallback)"
+        std::dynamic_pointer_cast<const core::TableScanNode>(childNode) != nullptr
+#ifdef GLUTEN_ENABLE_GPU
+            || std::dynamic_pointer_cast<const CudfValueStreamNode>(childNode) != nullptr
+#endif
+        ) && childNode->outputType()->size() > requiredChildOutput.size(),
+        "injectedProject is true, but the ProjectNode or TableScanNode or CudfValueStreamNode (in case of projection fallback)"
         " is missing or does not have the corresponding projection field");
 
     bool isStack = generateRel.has_advanced_extension() &&
@@ -858,18 +969,18 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
   }
 
   std::optional<std::string> ordinalityName = std::nullopt;
-  std::optional<std::string> emptyUnnestValueName = std::nullopt;
+  std::optional<std::string> markerName = std::nullopt;
   if (generateRel.has_advanced_extension()) {
     if (SubstraitParser::configSetInOptimization(generateRel.advanced_extension(), "isPosExplode=")) {
       ordinalityName = std::make_optional<std::string>("pos");
     }
     if (SubstraitParser::configSetInOptimization(generateRel.advanced_extension(), "isOuter=")) {
-      emptyUnnestValueName = std::make_optional<std::string>("empty_unnest");
+      markerName = std::make_optional<std::string>("marker");
     }
   }
 
   return std::make_shared<core::UnnestNode>(
-      nextPlanNodeId(), replicated, unnest, std::move(unnestNames), ordinalityName, emptyUnnestValueName, childNode);
+      nextPlanNodeId(), replicated, unnest, std::move(unnestNames), ordinalityName, markerName, childNode);
 }
 
 const core::WindowNode::Frame SubstraitToVeloxPlanConverter::createWindowFrame(
@@ -1000,28 +1111,15 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
     }
   }
 
-  if (windowRel.has_advanced_extension() &&
-      SubstraitParser::configSetInOptimization(windowRel.advanced_extension(), "isStreaming=")) {
-    return std::make_shared<core::WindowNode>(
-        nextPlanNodeId(),
-        partitionKeys,
-        sortingKeys,
-        sortingOrders,
-        windowColumnNames,
-        windowNodeFunctions,
-        true /*inputsSorted*/,
-        childNode);
-  } else {
-    return std::make_shared<core::WindowNode>(
-        nextPlanNodeId(),
-        partitionKeys,
-        sortingKeys,
-        sortingOrders,
-        windowColumnNames,
-        windowNodeFunctions,
-        false /*inputsSorted*/,
-        childNode);
-  }
+  return std::make_shared<core::WindowNode>(
+      nextPlanNodeId(),
+      partitionKeys,
+      sortingKeys,
+      sortingOrders,
+      windowColumnNames,
+      windowNodeFunctions,
+      true /*inputsSorted*/,
+      childNode);
 }
 
 core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(
@@ -1184,6 +1282,55 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
 }
 
 core::PlanNodePtr SubstraitToVeloxPlanConverter::constructValueStreamNode(
+    const ::substrait::ReadRel& readRel, int32_t streamIdx) {
+  // Use TableScanNode with iterator connector for runtime iterator inputs
+  // Get output schema from ReadRel
+  uint64_t colNum = 0;
+  std::vector<TypePtr> veloxTypeList;
+  if (readRel.has_base_schema()) {
+    const auto& baseSchema = readRel.base_schema();
+    colNum = baseSchema.names().size();
+    veloxTypeList = SubstraitParser::parseNamedStruct(baseSchema);
+  }
+
+  auto nodeId = ValueStreamConnectorFactory::nodeIdOf(streamIdx);
+  std::vector<std::string> outNames;
+  outNames.reserve(colNum);
+  for (int idx = 0; idx < colNum; idx++) {
+    // TODO: We'd use the designated names in readRel rather than assigning new names.
+    auto colName = fmt::format("node_{}_{}", nodeId, idx);
+    outNames.emplace_back(colName);
+  }
+  auto outputType = ROW(std::move(outNames), std::move(veloxTypeList));
+
+  // Create TableHandle
+  auto tableHandle = std::make_shared<ValueStreamTableHandle>(kIteratorConnectorId);
+
+  // Create column assignments
+  connector::ColumnHandleMap assignments;
+  for (int idx = 0; idx < outputType->size(); idx++) {
+    auto name = outputType->nameOf(idx);
+    auto type = outputType->childAt(idx);
+    assignments[name] = std::make_shared<ValueStreamColumnHandle>(name, type);
+  }
+
+  // Create TableScanNode
+  auto tableScanNode = std::make_shared<core::TableScanNode>(
+      nodeId,
+      outputType,
+      tableHandle,
+      assignments);
+
+  // Mark this as a stream-based split
+  auto splitInfo = std::make_shared<SplitInfo>();
+  splitInfo->leafType = SplitInfo::LeafType::SPLIT_AWARE_STREAM;
+  splitInfoMap_[tableScanNode->id()] = splitInfo;
+
+  return tableScanNode;
+}
+
+#ifdef GLUTEN_ENABLE_GPU
+core::PlanNodePtr SubstraitToVeloxPlanConverter::constructCudfValueStreamNode(
     const ::substrait::ReadRel& readRel,
     int32_t streamIdx) {
   // Get the input schema of this iterator.
@@ -1208,30 +1355,33 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::constructValueStreamNode(
   std::shared_ptr<ResultIterator> iterator;
   if (!validationMode_) {
     VELOX_CHECK_LT(streamIdx, inputIters_.size(), "Could not find stream index {} in input iterator list.", streamIdx);
-    iterator = inputIters_[streamIdx];
+    iterator = std::move(inputIters_[streamIdx]);
   }
-  auto node = std::make_shared<ValueStreamNode>(nextPlanNodeId(), outputType, std::move(iterator));
+  auto node = std::make_shared<CudfValueStreamNode>(nextPlanNodeId(), outputType, std::move(iterator));
 
   auto splitInfo = std::make_shared<SplitInfo>();
-  splitInfo->isStream = true;
+  splitInfo->leafType = SplitInfo::LeafType::TRIVIAL_LEAF;
   splitInfoMap_[node->id()] = splitInfo;
   return node;
 }
+#endif
 
 core::PlanNodePtr SubstraitToVeloxPlanConverter::constructValuesNode(
     const ::substrait::ReadRel& readRel,
     int32_t streamIdx) {
-  std::vector<RowVectorPtr> values;
+  // ValuesNode is only used for validation/benchmarking with query trace
+  // It loads all data from the iterator at plan construction time
   VELOX_CHECK_LT(streamIdx, inputIters_.size(), "Could not find stream index {} in input iterator list.", streamIdx);
-  const auto iterator = inputIters_[streamIdx];
-  while (iterator->hasNext()) {
-    auto cb = VeloxColumnarBatch::from(defaultLeafVeloxMemoryPool().get(), iterator->next());
-    values.emplace_back(cb->getRowVector());
+  const auto iter = std::move(inputIters_[streamIdx]);
+  std::vector<RowVectorPtr> rowVectors;
+  while (iter->hasNext()) {
+    auto batch = iter->next();
+    auto veloxBatch = VeloxColumnarBatch::from(defaultLeafVeloxMemoryPool().get(), batch);
+    rowVectors.emplace_back(veloxBatch->getRowVector());
   }
-  auto node = std::make_shared<facebook::velox::core::ValuesNode>(nextPlanNodeId(), std::move(values));
-
+  auto node = std::make_shared<facebook::velox::core::ValuesNode>(nextPlanNodeId(), std::move(rowVectors));
   auto splitInfo = std::make_shared<SplitInfo>();
-  splitInfo->isStream = true;
+  splitInfo->leafType = SplitInfo::LeafType::TRIVIAL_LEAF;
   splitInfoMap_[node->id()] = splitInfo;
   return node;
 }
@@ -1244,19 +1394,25 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
         !readRel.common().has_emit(), "Emit not supported for ValuesNode and TableScanNode related Substrait plans.");
   }
 
-  // Check if the ReadRel specifies an input of stream. If yes, build ValueStreamNode as the data source.
   auto streamIdx = getStreamIndex(readRel);
   if (streamIdx >= 0) {
-    // Only used in benchmark enable query trace, replace ValueStreamNode to ValuesNode to support serialization.
-    if (LIKELY(!veloxCfg_->get<bool>(kQueryTraceEnabled, false))) {
-      return constructValueStreamNode(readRel, streamIdx);
-    } else {
+    // Check if the ReadRel specifies an input of stream. If yes, build TableScanNode with iterator connector.
+    const bool isQueryTraceEnabled = veloxCfg_->get<bool>(kQueryTraceEnabled, false);
+    if (isQueryTraceEnabled) {
+      // Only used in benchmark enable query trace, replace ValueStreamNode to ValuesNode to support serialization.
       return constructValuesNode(readRel, streamIdx);
     }
+#ifdef GLUTEN_ENABLE_GPU
+    if (veloxCfg_->get<bool>(kCudfEnabled, kCudfEnabledDefault)) {
+      return constructCudfValueStreamNode(readRel, streamIdx);
+    }
+#endif
+    return constructValueStreamNode(readRel, streamIdx);
   }
 
   // Otherwise, will create TableScan node for ReadRel.
   auto splitInfo = std::make_shared<SplitInfo>();
+  splitInfo->leafType = SplitInfo::LeafType::TABLE_SCAN;
   if (!validationMode_) {
     VELOX_CHECK_LT(splitInfoIdx_, splitInfos_.size(), "Plan must have readRel and related split info.");
     splitInfo = splitInfos_[splitInfoIdx_++];
@@ -1286,23 +1442,24 @@ core::PlanNodePtr SubstraitToVeloxPlanConverter::toVeloxPlan(const ::substrait::
   bool filterPushdownEnabled = true;
   auto names = colNameList;
   auto types = veloxTypeList;
-  auto dataColumns = ROW(std::move(names), std::move(types));
-  std::shared_ptr<connector::hive::HiveTableHandle> tableHandle;
-  if (!readRel.has_filter()) {
-    tableHandle = std::make_shared<connector::hive::HiveTableHandle>(
-        kHiveConnectorId, "hive_table", filterPushdownEnabled, common::SubfieldFilters{}, nullptr, dataColumns);
-  } else {
-    common::SubfieldFilters subfieldFilters;
-    auto remainingFilter = exprConverter_->toVeloxExpr(readRel.filter(), dataColumns);
 
-    tableHandle = std::make_shared<connector::hive::HiveTableHandle>(
-        kHiveConnectorId,
-        "hive_table",
-        filterPushdownEnabled,
-        std::move(subfieldFilters),
-        remainingFilter,
-        dataColumns);
+  // The columns we project from the file.
+  auto baseSchema = ROW(std::move(names), std::move(types));
+  // The columns present in the table, if not available default to the baseSchema.
+  auto tableSchema = splitInfo->tableSchema ? splitInfo->tableSchema : baseSchema;
+
+  connector::ConnectorTableHandlePtr tableHandle;
+  auto remainingFilter = readRel.has_filter() ? exprConverter_->toVeloxExpr(readRel.filter(), baseSchema) : nullptr;
+  auto connectorId = kHiveConnectorId;
+  if (useCudfTableHandle(splitInfos_) && veloxCfg_->get<bool>(kCudfEnableTableScan, kCudfEnableTableScanDefault) &&
+      veloxCfg_->get<bool>(kCudfEnabled, kCudfEnabledDefault)) {
+#ifdef GLUTEN_ENABLE_GPU
+    connectorId = kCudfHiveConnectorId;
+#endif
   }
+  common::SubfieldFilters subfieldFilters;
+  tableHandle = std::make_shared<connector::hive::HiveTableHandle>(
+      connectorId, "hive_table", filterPushdownEnabled, std::move(subfieldFilters), remainingFilter, tableSchema);
 
   // Get assignments and out names.
   std::vector<std::string> outNames;
@@ -1466,6 +1623,11 @@ void SubstraitToVeloxPlanConverter::constructFunctionMap(const ::substrait::Plan
   exprConverter_ = std::make_unique<SubstraitVeloxExprConverter>(pool_, functionMap_);
 }
 
+void SubstraitToVeloxPlanConverter::constructFunctionMap(std::unordered_map<uint64_t, std::string> substraitPlan) {
+  functionMap_ = std::move(substraitPlan);
+  exprConverter_ = std::make_unique<SubstraitVeloxExprConverter>(pool_, functionMap_);
+}
+
 std::string SubstraitToVeloxPlanConverter::findFuncSpec(uint64_t id) {
   return SubstraitParser::findFunctionSpec(functionMap_, id);
 }
