@@ -47,6 +47,7 @@ class VeloxSubstraitRoundTripTest : public OperatorTestBase {
   /// @param batchSize The batch Size of the data.
   std::vector<RowVectorPtr> makeVectors(int64_t size, int64_t childSize, int64_t batchSize) {
     std::vector<RowVectorPtr> vectors;
+    vectors.reserve(size);
     std::mt19937 gen(std::mt19937::default_seed);
     for (int i = 0; i < size; i++) {
       std::vector<VectorPtr> children;
@@ -71,7 +72,7 @@ class VeloxSubstraitRoundTripTest : public OperatorTestBase {
     auto veloxCfg =
         std::make_shared<facebook::velox::config::ConfigBase>(std::unordered_map<std::string, std::string>());
     std::shared_ptr<SubstraitToVeloxPlanConverter> substraitConverter_ =
-        std::make_shared<SubstraitToVeloxPlanConverter>(pool_.get(), veloxCfg.get(), std::nullopt, std::nullopt, true);
+        std::make_shared<SubstraitToVeloxPlanConverter>(pool_.get(), veloxCfg.get(), std::vector<std::shared_ptr<ResultIterator>>(), std::nullopt, std::nullopt, true);
 
     // Convert Substrait Plan to the same Velox Plan.
     auto samePlan = substraitConverter_->toVeloxPlan(substraitPlan);
@@ -93,7 +94,7 @@ class VeloxSubstraitRoundTripTest : public OperatorTestBase {
           std::make_shared<facebook::velox::config::ConfigBase>(std::unordered_map<std::string, std::string>());
       std::shared_ptr<SubstraitToVeloxPlanConverter> substraitConverter_ =
           std::make_shared<SubstraitToVeloxPlanConverter>(
-              pool_.get(), veloxCfg.get(), std::nullopt, std::nullopt, true);
+              pool_.get(), veloxCfg.get(), std::vector<std::shared_ptr<ResultIterator>>(), std::nullopt, std::nullopt, true);
       // Convert Substrait Plan to the same Velox Plan.
       auto samePlan = substraitConverter_->toVeloxPlan(substraitPlan);
 
@@ -477,7 +478,7 @@ TEST_F(VeloxSubstraitRoundTripTest, sumAndCountCompanion) {
   auto plan = PlanBuilder()
                   .values(vectors)
                   .singleAggregation({}, {"sum_partial(c1)", "count_partial(c4)"})
-                  .singleAggregation({}, {"sum_merge_extract(a0)", "count_merge_extract(a1)"})
+                  .singleAggregation({}, {"sum_merge_extract_bigint(a0)", "count_merge_extract_bigint(a1)"})
                   .planNode();
 
   assertPlanConversion(plan, "SELECT sum(c1), count(c4) FROM tmp");
@@ -492,7 +493,7 @@ TEST_F(VeloxSubstraitRoundTripTest, sumGlobalCompanion) {
                   .values(vectors)
                   .singleAggregation({"c0"}, {"sum_partial(c0)", "sum_partial(c1)"})
                   .singleAggregation({"c0"}, {"sum_merge(a0)", "sum_merge(a1)"})
-                  .singleAggregation({"c0"}, {"sum_merge_extract(a0)", "sum_merge_extract(a1)"})
+                  .singleAggregation({"c0"}, {"sum_merge_extract_bigint(a0)", "sum_merge_extract_bigint(a1)"})
                   .planNode();
   assertPlanConversion(plan, "SELECT c0, sum(c0), sum(c1) FROM tmp GROUP BY c0");
 }
@@ -505,7 +506,7 @@ TEST_F(VeloxSubstraitRoundTripTest, sumMaskCompanion) {
                   .values(vectors)
                   .project({"c0", "c1", "c2 % 2 < 10 AS m0", "c3 % 3 = 0 AS m1"})
                   .singleAggregation({}, {"sum_partial(c0)", "sum_partial(c0)", "sum_partial(c1)"}, {"m0", "m1", "m1"})
-                  .singleAggregation({}, {"sum_merge_extract(a0)", "sum_merge_extract(a1)", "sum_merge_extract(a2)"})
+                  .singleAggregation({}, {"sum_merge_extract_bigint(a0)", "sum_merge_extract_bigint(a1)", "sum_merge_extract_bigint(a2)"})
                   .planNode();
 
   assertPlanConversion(
@@ -538,7 +539,7 @@ TEST_F(VeloxSubstraitRoundTripTest, avgCompanion) {
   auto plan = PlanBuilder()
                   .values(vectors)
                   .singleAggregation({}, {"avg_partial(c4)"})
-                  .singleAggregation({}, {"avg_merge_extract(a0)"})
+                  .singleAggregation({}, {"avg_merge_extract_double(a0)"})
                   .planNode();
 
   assertPlanConversion(plan, "SELECT avg(c4) FROM tmp");
