@@ -18,7 +18,7 @@
 ####################################################################################################
 #  The main function of this script is to allow developers to build the environment with one click #
 #  Recommended commands for first-time installation:                                               #
-#  ./dev/buildbundle-veloxbe.sh                                                            #
+#  ./dev/buildbundle-veloxbe.sh                                                                    #
 ####################################################################################################
 set -exu
 
@@ -140,15 +140,15 @@ do
         shift # Remove argument name from processing
         ;;
         --build_arrow=*)
-        BUILD_ARROW=("${arg#*=}")
+        BUILD_ARROW="${arg#*=}"
         shift # Remove argument name from processing
         ;;
         --num_threads=*)
-        NUM_THREADS=("${arg#*=}")
+        NUM_THREADS="${arg#*=}"
         shift # Remove argument name from processing
         ;;
         --spark_version=*)
-        SPARK_VERSION=("${arg#*=}")
+        SPARK_VERSION="${arg#*=}"
         shift # Remove argument name from processing
         ;;
 	      *)
@@ -159,9 +159,7 @@ do
 done
 
 if [[ "$(uname)" == "Darwin" ]]; then
-    INSTALL_PREFIX=${INSTALL_PREFIX:-${VELOX_HOME}/deps-install}
-else
-    INSTALL_PREFIX=${INSTALL_PREFIX:-"/usr/local"}
+    export INSTALL_PREFIX=${INSTALL_PREFIX:-${VELOX_HOME}/deps-install}
 fi
 
 function concat_velox_param {
@@ -195,27 +193,39 @@ if [ "$ENABLE_VCPKG" = "ON" ]; then
     source ./dev/vcpkg/env.sh ${BUILD_OPTIONS}
 fi
 
-if [ "$SPARK_VERSION" = "3.2" ] || [ "$SPARK_VERSION" = "3.3" ] \
-  || [ "$SPARK_VERSION" = "3.4" ] || [ "$SPARK_VERSION" = "3.5" ] \
-  || [ "$SPARK_VERSION" = "ALL" ]; then
+# Supported Spark versions
+SUPPORTED_SPARK_VERSIONS=("3.3" "3.4" "3.5" "4.0" "4.1" "ALL")
+
+# Check if SPARK_VERSION is in the supported list
+pattern=" $SPARK_VERSION "
+if [[ " ${SUPPORTED_SPARK_VERSIONS[*]} " =~ $pattern ]]; then
   echo "Building for Spark $SPARK_VERSION"
 else
   echo "Invalid Spark version: $SPARK_VERSION"
+  echo "Supported versions: 3.3 3.4 3.5 4.0 4.1 ALL"
   exit 1
 fi
 
 concat_velox_param
 
 function build_arrow {
+  if [ ! -d "$VELOX_HOME" ]; then
+    get_velox
+    if [ -z "${GLUTEN_VCPKG_ENABLED:-}" ] && [ $RUN_SETUP_SCRIPT == "ON" ]; then
+      setup_dependencies
+    else
+      setup_dependencies_arrow
+    fi
+  fi
   cd $GLUTEN_DIR/dev
-  source ./build_arrow.sh
+  source ./build-arrow.sh
 }
 
 function build_velox {
   echo "Start to build Velox"
   cd $GLUTEN_DIR/ep/build-velox/src
   # When BUILD_TESTS is on for gluten cpp, we need turn on VELOX_BUILD_TEST_UTILS via build_test_utils.
-  ./build_velox.sh --enable_s3=$ENABLE_S3 --enable_gcs=$ENABLE_GCS --build_type=$BUILD_TYPE --enable_hdfs=$ENABLE_HDFS \
+  ./build-velox.sh --enable_s3=$ENABLE_S3 --enable_gcs=$ENABLE_GCS --build_type=$BUILD_TYPE --enable_hdfs=$ENABLE_HDFS \
                    --enable_abfs=$ENABLE_ABFS --enable_gpu=$ENABLE_GPU --build_test_utils=$BUILD_TESTS \
                    --build_tests=$BUILD_VELOX_TESTS --build_benchmarks=$BUILD_VELOX_BENCHMARKS --num_threads=$NUM_THREADS \
                    --velox_home=$VELOX_HOME
@@ -241,6 +251,7 @@ function build_gluten_cpp {
     -DENABLE_HDFS=$ENABLE_HDFS \
     -DENABLE_ABFS=$ENABLE_ABFS \
     -DENABLE_GPU=$ENABLE_GPU \
+    -DCMAKE_EXPORT_COMPILE_COMMANDS=ON \
     -DENABLE_ENHANCED_FEATURES=$ENABLE_ENHANCED_FEATURES"
 
   if [ $OS == 'Darwin' ]; then
@@ -259,19 +270,18 @@ function build_velox_backend {
   build_gluten_cpp
 }
 
-(
+function get_velox {
   cd $GLUTEN_DIR/ep/build-velox/src
-  ./get_velox.sh $VELOX_PARAMETER
-)
+  ./get-velox.sh $VELOX_PARAMETER
+}
 
-OS=`uname -s`
-ARCH=`uname -m`
-DEPENDENCY_DIR=${DEPENDENCY_DIR:-$CURRENT_DIR/../ep/_ep}
-mkdir -p ${DEPENDENCY_DIR}
+function setup_dependencies {
+  DEPENDENCY_DIR=${DEPENDENCY_DIR:-$CURRENT_DIR/../ep/_ep}
+  mkdir -p ${DEPENDENCY_DIR}
+
+  source $GLUTEN_DIR/dev/build-helper-functions.sh
+  source ${VELOX_HOME}/scripts/setup-common.sh
 
-source $GLUTEN_DIR/dev/build_helper_functions.sh
-source ${VELOX_HOME}/scripts/setup-common.sh
-if [ -z "${GLUTEN_VCPKG_ENABLED:-}" ] && [ $RUN_SETUP_SCRIPT == "ON" ]; then
   echo "Start to install dependencies"
   pushd $VELOX_HOME
   if [ $OS == 'Linux' ]; then
@@ -286,22 +296,35 @@ if [ -z "${GLUTEN_VCPKG_ENABLED:-}" ] && [ $RUN_SETUP_SCRIPT == "ON" ]; then
     install_aws_deps
   fi
   if [ $ENABLE_GCS == "ON" ]; then
-    install_gcs-sdk-cpp
+    install_gcs_sdk_cpp
   fi
   if [ $ENABLE_ABFS == "ON" ]; then
     export AZURE_SDK_DISABLE_AUTO_VCPKG=ON
-    install_azure-storage-sdk-cpp
+    install_azure_storage_sdk_cpp
   fi
   popd
-fi
+}
 
-commands_to_run=${OTHER_ARGUMENTS:-}
+function setup_dependencies_arrow {
+  DEPENDENCY_DIR=${DEPENDENCY_DIR:-$CURRENT_DIR/../ep/_ep}
+  mkdir -p ${DEPENDENCY_DIR}
+
+  source ${VELOX_HOME}/scripts/setup-common.sh
+}
+
+OS=`uname -s`
+ARCH=`uname -m`
+commands_to_run=(${OTHER_ARGUMENTS[@]:-})
 (
-  if [[ "x$commands_to_run" == "x" ]]; then
+  if [[ ${#commands_to_run[@]} -eq 0 ]]; then
+    get_velox
+    if [ -z "${GLUTEN_VCPKG_ENABLED:-}" ] && [ $RUN_SETUP_SCRIPT == "ON" ]; then
+      setup_dependencies
+    fi
     build_velox_backend
   else
-    echo "Commands to run: $commands_to_run"
-    for cmd in "$commands_to_run"; do
+    echo "Commands to run: ${commands_to_run[@]}"
+    for cmd in "${commands_to_run[@]}"; do
        "${cmd}"
     done
   fi
