@@ -31,7 +31,7 @@ fi
 
 # build gluten with velox backend, prompt always respond y
 export PROMPT_ALWAYS_RESPOND=y
-./dev/buildbundle-veloxbe.sh --enable_vcpkg=OFF --build_tests=ON --build_benchmarks=ON --enable_s3=ON --enable_hdfs=ON "$@"
+./dev/buildbundle-veloxbe.sh --enable_vcpkg=OFF --build_tests=OFF --build_benchmarks=ON --enable_s3=ON --enable_hdfs=ON "$@"
 
 # make thirdparty package
 ./dev/build-thirdparty.sh
