@@ -21,7 +21,8 @@ import org.apache.gluten.runtime.Runtime;
 import org.apache.gluten.runtime.Runtimes;
 import org.apache.gluten.utils.ArrowAbiUtil;
 import org.apache.gluten.utils.ArrowUtil;
-import org.apache.gluten.utils.InternalRowUtl;
+import org.apache.gluten.utils.InternalRowUtil;
+import org.apache.gluten.vectorized.ArrowColumnarBatch;
 import org.apache.gluten.vectorized.ArrowWritableColumnVector;
 
 import com.google.common.annotations.VisibleForTesting;
@@ -105,10 +106,7 @@ public final class ColumnarBatches {
     return identifyBatchType(batch) == BatchType.ZERO_COLUMN;
   }
 
-  /**
-   * This method will always return a velox based ColumnarBatch. This method will close the input
-   * column batch.
-   */
+  /** This method will always return a velox based ColumnarBatch. */
   public static ColumnarBatch select(String backendName, ColumnarBatch batch, int[] columnIndices) {
     final Runtime runtime = Runtimes.contextInstance(backendName, "ColumnarBatches#select");
     switch (identifyBatchType(batch)) {
@@ -171,6 +169,15 @@ public final class ColumnarBatches {
     }
   }
 
+  public static ArrowColumnarBatch convertToArrowColumnarBatch(ColumnarBatch sparkColumnarBatch) {
+    int numCols = sparkColumnarBatch.numCols();
+    ArrowWritableColumnVector[] writableColumns = new ArrowWritableColumnVector[numCols];
+    for (int i = 0; i < numCols; i++) {
+      writableColumns[i] = (ArrowWritableColumnVector) sparkColumnarBatch.column(i);
+    }
+    return new ArrowColumnarBatch(writableColumns, sparkColumnarBatch.numRows());
+  }
+
   public static ColumnarBatch load(BufferAllocator allocator, ColumnarBatch input) {
     if (isZeroColumnBatch(input)) {
       return input;
@@ -405,7 +412,7 @@ public final class ColumnarBatches {
   public static String toString(ColumnarBatch batch, int start, int length) {
     ColumnarBatch loadedBatch = ensureLoaded(ArrowBufferAllocators.contextInstance(), batch);
     StructType type = SparkArrowUtil.fromArrowSchema(ArrowUtil.toSchema(loadedBatch));
-    return InternalRowUtl.toString(
+    return InternalRowUtil.toString(
         type,
         JavaConverters.<InternalRow>asScalaIterator(loadedBatch.rowIterator()),
         start,
