@@ -28,7 +28,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.nio.charset.StandardCharsets;
-import java.util.List;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicInteger;
 
 public class NativePlanEvaluator {
@@ -43,6 +43,12 @@ public class NativePlanEvaluator {
     this.jniWrapper = PlanEvaluatorJniWrapper.create(runtime);
   }
 
+  public static NativePlanEvaluator create(String backendName, Map<String, String> extraConf) {
+    return new NativePlanEvaluator(
+        Runtimes.contextInstance(
+            backendName, String.format("NativePlanEvaluator-%d", id.getAndIncrement()), extraConf));
+  }
+
   public static NativePlanEvaluator create(String backendName) {
     return new NativePlanEvaluator(
         Runtimes.contextInstance(
@@ -53,6 +59,10 @@ public class NativePlanEvaluator {
     return jniWrapper.nativeValidateWithFailureReason(subPlan);
   }
 
+  public boolean doNativeValidateExpression(byte[] expression, byte[] inputType, byte[][] mapping) {
+    return jniWrapper.nativeValidateExpression(expression, inputType, mapping);
+  }
+
   public static void injectWriteFilesTempPath(String path, String fileName) {
     PlanEvaluatorJniWrapper.injectWriteFilesTempPath(
         path.getBytes(StandardCharsets.UTF_8), fileName.getBytes(StandardCharsets.UTF_8));
@@ -60,25 +70,24 @@ public class NativePlanEvaluator {
 
   // Used by WholeStageTransform to create the native computing pipeline and
   // return a columnar result iterator.
+  // Supports both creation-time splits (splitInfo, iterList) and runtime splits (via addSplits()).
   public ColumnarBatchOutIterator createKernelWithBatchIterator(
       byte[] wsPlan,
       byte[][] splitInfo,
-      List<ColumnarBatchInIterator> iterList,
+      ColumnarBatchInIterator[] iterList,
       int partitionIndex,
-      String spillDirPath,
-      boolean enableCudf)
+      String spillDirPath)
       throws RuntimeException {
     final long itrHandle =
         jniWrapper.nativeCreateKernelWithIterator(
             wsPlan,
             splitInfo,
-            iterList.toArray(new ColumnarBatchInIterator[0]),
+            iterList,
             TaskContext.get().stageId(),
             partitionIndex, // TaskContext.getPartitionId(),
             TaskContext.get().taskAttemptId(),
             DebugUtil.isDumpingEnabledForTask(),
-            spillDirPath,
-            enableCudf);
+            spillDirPath);
     final ColumnarBatchOutIterator out = createOutIterator(runtime, itrHandle);
     runtime
         .memoryManager()
