@@ -53,7 +53,8 @@ object NativeMemoryManager {
       rl,
       ConfigUtil.serialize(
         GlutenConfig
-          .getNativeSessionConf(backendName, GlutenConfigUtil.parseConfig(SQLConf.get.getAllConfs)))
+          .getNativeSessionConf(backendName, GlutenConfigUtil.parseConfig(SQLConf.get.getAllConfs))
+          .asJava)
     )
     spillers.append(new Spiller() {
       override def spill(self: MemoryTarget, phase: Spiller.Phase, size: Long): Long = phase match {
@@ -111,8 +112,8 @@ object NativeMemoryManager {
     }
     override def priority(): Int = {
       // Memory managers should be released after all runtimes are released.
-      // So lower the priority to 0.
-      0
+      // So set the priority lower than runtime resources.
+      10
     }
     override def resourceName(): String = "nmm"
   }
