@@ -16,14 +16,12 @@
  */
 package org.apache.spark.shuffle.gluten.celeborn;
 
-import org.apache.gluten.backendsapi.BackendsApiManager;
 import org.apache.gluten.config.GlutenConfig;
 import org.apache.gluten.exception.GlutenException;
 import org.apache.gluten.shuffle.NeedCustomColumnarBatchSerializer;
 import org.apache.gluten.shuffle.SupportsColumnarShuffle;
 
 import com.google.common.base.Preconditions;
-import com.google.common.collect.Iterators;
 import org.apache.celeborn.client.LifecycleManager;
 import org.apache.celeborn.client.ShuffleClient;
 import org.apache.celeborn.common.CelebornConf;
@@ -36,12 +34,9 @@ import org.slf4j.LoggerFactory;
 
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
-import java.util.Arrays;
-import java.util.List;
-import java.util.Map;
+import java.util.Iterator;
 import java.util.ServiceLoader;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.stream.Collectors;
 
 public class CelebornShuffleManager
     implements ShuffleManager, SupportsColumnarShuffle, NeedCustomColumnarBatchSerializer {
@@ -68,48 +63,25 @@ public class CelebornShuffleManager
   private static final CelebornColumnarBatchSerializerFactory columnarBatchSerializerFactory;
 
   static {
-    final ServiceLoader<CelebornShuffleWriterFactory> loader =
-        ServiceLoader.load(CelebornShuffleWriterFactory.class);
-    final List<CelebornShuffleWriterFactory> factoryList =
-        Arrays.stream(Iterators.toArray(loader.iterator(), CelebornShuffleWriterFactory.class))
-            .collect(Collectors.toList());
+    final Iterator<CelebornShuffleWriterFactory> shuffleWriterFactoryIterator =
+        ServiceLoader.load(CelebornShuffleWriterFactory.class).iterator();
     Preconditions.checkState(
-        !factoryList.isEmpty(), "No factory found for Celeborn shuffle writer");
-    final Map<String, CelebornShuffleWriterFactory> factoryMap =
-        factoryList.stream()
-            .collect(Collectors.toMap(CelebornShuffleWriterFactory::backendName, f -> f));
+        shuffleWriterFactoryIterator.hasNext(), "No Celeborn shuffle writer factory found");
+    writerFactory = shuffleWriterFactoryIterator.next();
+    Preconditions.checkState(
+        !shuffleWriterFactoryIterator.hasNext(),
+        "Multiple factories found for Celeborn shuffle writer");
 
-    final String backendName = BackendsApiManager.getBackendName();
-    if (!factoryMap.containsKey(backendName)) {
-      throw new UnsupportedOperationException(
-          "No Celeborn shuffle writer factory found for backend " + backendName);
-    }
-    writerFactory = factoryMap.get(backendName);
+    final Iterator<CelebornColumnarBatchSerializerFactory> serializerFactoryIterator =
+        ServiceLoader.load(CelebornColumnarBatchSerializerFactory.class).iterator();
 
-    final ServiceLoader<CelebornColumnarBatchSerializerFactory>
-        celebornColumnarBatchSerializerFactoriesLoader =
-            ServiceLoader.load(CelebornColumnarBatchSerializerFactory.class);
-    final List<CelebornColumnarBatchSerializerFactory> columnarBatchSerializerFactoryList =
-        Arrays.stream(
-                Iterators.toArray(
-                    celebornColumnarBatchSerializerFactoriesLoader.iterator(),
-                    CelebornColumnarBatchSerializerFactory.class))
-            .collect(Collectors.toList());
-    //   for now, we ignore check since CH backend has not support this feature yet.
-    //    Preconditions.checkState(
-    //        !columnarBatchSerializerFactoryList.isEmpty(),
-    //        "No factory found for Celeborn columnar batch serializer");
-    final Map<String, CelebornColumnarBatchSerializerFactory> columanrBatchSerilizerFactoryMap =
-        columnarBatchSerializerFactoryList.stream()
-            .collect(Collectors.toMap(CelebornColumnarBatchSerializerFactory::backendName, f -> f));
-
-    //   for now, we ignore check since CH backend has not support this feature yet.
-    //    if (!columanrBatchSerilizerFactoryMap.containsKey(backendName)) {
-    //      throw new UnsupportedOperationException(
-    //          "No Celeborn columnar batch serializer writer factory found for backend " +
-    // backendName);
-    //    }
-    columnarBatchSerializerFactory = columanrBatchSerilizerFactoryMap.get(backendName);
+    Preconditions.checkState(
+        serializerFactoryIterator.hasNext(),
+        "No factory found for Celeborn columnar batch serializer");
+    columnarBatchSerializerFactory = serializerFactoryIterator.next();
+    Preconditions.checkState(
+        !serializerFactoryIterator.hasNext(),
+        "Multiple factories found for Celeborn columnar batch serializer");
   }
 
   private final SparkConf conf;
@@ -309,7 +281,8 @@ public class CelebornShuffleManager
       _vanillaCelebornShuffleManager = null;
     }
     if (failedShuffleCleaner != null) {
-      CelebornUtils.resetFailedShuffleCleaner(failedShuffleCleaner);
+      CelebornUtils.stopFailedShuffleCleaner(failedShuffleCleaner);
+      failedShuffleCleaner = null;
     }
   }
 
