@@ -18,7 +18,7 @@ package org.apache.gluten.extension.caller
 
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanExec
 import org.apache.spark.sql.execution.columnar.InMemoryRelation
-import org.apache.spark.sql.execution.streaming.StreamExecution
+import org.apache.spark.util.SparkVersionUtil
 
 /**
  * Helper API that stores information about the call site of the columnar rule. Specific columnar
@@ -30,6 +30,7 @@ trait CallerInfo {
   def isAqe(): Boolean
   def isCache(): Boolean
   def isStreaming(): Boolean
+  def isBloomFilterStatFunction(): Boolean
 }
 
 object CallerInfo {
@@ -41,7 +42,8 @@ object CallerInfo {
   private class Impl(
       override val isAqe: Boolean,
       override val isCache: Boolean,
-      override val isStreaming: Boolean
+      override val isStreaming: Boolean,
+      override val isBloomFilterStatFunction: Boolean
   ) extends CallerInfo
 
   /*
@@ -55,7 +57,8 @@ object CallerInfo {
     new Impl(
       isAqe = inAqeCall(stack),
       isCache = inCacheCall(stack),
-      isStreaming = inStreamingCall(stack))
+      isStreaming = inStreamingCall(stack),
+      isBloomFilterStatFunction = inBloomFilterStatFunctionCall(stack))
   }
 
   private def inAqeCall(stack: Seq[StackTraceElement]): Boolean = {
@@ -67,14 +70,29 @@ object CallerInfo {
   }
 
   private def inStreamingCall(stack: Seq[StackTraceElement]): Boolean = {
-    stack.exists(_.getClassName.equals(StreamExecution.getClass.getName.split('$').head))
+    val streamName = if (SparkVersionUtil.gteSpark41) {
+      "org.apache.spark.sql.execution.streaming.runtime.StreamExecution"
+    } else {
+      "org.apache.spark.sql.execution.streaming.StreamExecution"
+    }
+    stack.exists(_.getClassName.equals(streamName))
+  }
+
+  private def inBloomFilterStatFunctionCall(stack: Seq[StackTraceElement]): Boolean = {
+    val res = stack.exists(
+      _.getClassName.equals("org.apache.spark.sql.DataFrameStatFunctions")
+        && stack.exists(_.getMethodName.equals("bloomFilter")))
+    res
   }
 
   /** For testing only. */
-  def withLocalValue[T](isAqe: Boolean, isCache: Boolean, isStreaming: Boolean = false)(
-      body: => T): T = {
+  def withLocalValue[T](
+      isAqe: Boolean,
+      isCache: Boolean,
+      isStreaming: Boolean = false,
+      isBloomFilterStatFunction: Boolean = false)(body: => T): T = {
     val prevValue = localStorage.get()
-    val newValue = new Impl(isAqe, isCache, isStreaming)
+    val newValue = new Impl(isAqe, isCache, isStreaming, isBloomFilterStatFunction)
     localStorage.set(Some(newValue))
     try {
       body
