@@ -26,6 +26,7 @@ import org.apache.commons.lang3.StringUtils
 
 import java.io.{File, IOException}
 import java.nio.file.Paths
+import java.util.concurrent.ConcurrentHashMap
 
 /**
  * Manages Gluten's local directories, for storing jars, libs, spill files, or other temporary
@@ -53,44 +54,39 @@ class SparkDirectoryUtil private (val roots: Array[String]) extends Logging {
       }
   }
 
-  private val NAMESPACE_MAPPING: java.util.Map[String, Namespace] = new java.util.HashMap()
+  private val NAMESPACE_MAPPING: java.util.Map[String, Namespace] =
+    new ConcurrentHashMap[String, Namespace]
 
-  def namespace(name: String): Namespace = synchronized {
-    if (NAMESPACE_MAPPING.containsKey(name)) {
-      return NAMESPACE_MAPPING.get(name)
-    }
-    // or create new
-    val namespace = new Namespace(ROOTS, name)
-    NAMESPACE_MAPPING.put(name, namespace)
-    namespace
-  }
+  def namespace(name: String): Namespace =
+    NAMESPACE_MAPPING.computeIfAbsent(name, (name: String) => new Namespace(ROOTS, name))
 }
 
 object SparkDirectoryUtil extends Logging {
-  private var INSTANCE: SparkDirectoryUtil = _
+  @volatile private var roots: Array[String] = _
+  private lazy val INSTANCE: SparkDirectoryUtil = {
+    if (this.roots == null) {
+      throw new IllegalStateException("SparkDirectoryUtil not initialized")
+    }
+    new SparkDirectoryUtil(this.roots)
+  }
 
-  def init(conf: SparkConf): Unit = synchronized {
+  def init(conf: SparkConf): Unit = {
     val roots = Utils.getConfiguredLocalDirs(conf)
     init(roots)
   }
 
   private def init(roots: Array[String]): Unit = synchronized {
-    if (INSTANCE == null) {
-      INSTANCE = new SparkDirectoryUtil(roots)
-      return
-    }
-    if (INSTANCE.roots.toSet != roots.toSet) {
+    if (this.roots == null) {
+      this.roots = roots
+    } else if (this.roots.toSet != roots.toSet) {
       throw new IllegalArgumentException(
-        s"Reinitialize SparkDirectoryUtil with different root dirs: old: ${INSTANCE.ROOTS
+        s"Reinitialize SparkDirectoryUtil with different root dirs: old: ${this.roots
             .mkString("Array(", ", ", ")")}, new: ${roots.mkString("Array(", ", ", ")")}"
       )
     }
   }
 
-  def get(): SparkDirectoryUtil = synchronized {
-    assert(INSTANCE != null, "Default instance of SparkDirectoryUtil was not set yet")
-    INSTANCE
-  }
+  def get(): SparkDirectoryUtil = INSTANCE
 }
 
 class Namespace(private val parents: Array[File], private val name: String) {
