@@ -16,13 +16,12 @@
  */
 package org.apache.flink.table.planner.plan.nodes.exec.common;
 
-import org.apache.gluten.table.runtime.operators.GlutenOneInputOperator;
-import org.apache.gluten.util.LogicalTypeConverter;
-import org.apache.gluten.util.PlanNodeIdGenerator;
+import org.apache.gluten.velox.VeloxSourceSinkFactory;
 
 import org.apache.flink.api.common.io.OutputFormat;
 import org.apache.flink.api.dag.Transformation;
 import org.apache.flink.api.java.typeutils.InputTypeConfigurable;
+import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.ReadableConfig;
 import org.apache.flink.runtime.state.KeyGroupRangeAssignment;
 import org.apache.flink.streaming.api.datastream.CustomSinkOperatorUidHashes;
@@ -71,6 +70,7 @@ import org.apache.flink.table.runtime.keyselector.RowDataKeySelector;
 import org.apache.flink.table.runtime.operators.sink.ConstraintEnforcer;
 import org.apache.flink.table.runtime.operators.sink.RowKindSetter;
 import org.apache.flink.table.runtime.operators.sink.SinkOperator;
+import org.apache.flink.table.runtime.operators.sink.StreamRecordTimestampInserter;
 import org.apache.flink.table.runtime.typeutils.InternalTypeInfo;
 import org.apache.flink.table.types.logical.BinaryType;
 import org.apache.flink.table.types.logical.CharType;
@@ -96,7 +96,6 @@ import java.util.stream.IntStream;
  */
 public abstract class CommonExecSink extends ExecNodeBase<Object>
     implements MultipleTransformationTranslator<Object> {
-
   public static final String CONSTRAINT_VALIDATOR_TRANSFORMATION = "constraint-validator";
   public static final String PARTITIONER_TRANSFORMATION = "partitioner";
   public static final String UPSERT_MATERIALIZE_TRANSFORMATION = "upsert-materialize";
@@ -206,16 +205,25 @@ public abstract class CommonExecSink extends ExecNodeBase<Object>
     if (targetRowKind.isPresent()) {
       sinkTransform = applyRowKindSetter(sinkTransform, targetRowKind.get(), config);
     }
-
-    return (Transformation<Object>)
-        applySinkProvider(
-            sinkTransform,
-            streamExecEnv,
-            runtimeProvider,
-            rowtimeFieldIndex,
-            sinkParallelism,
-            config,
-            classLoader);
+    // --- Begin Gluten-specific code changes ---
+    Transformation<Object> transformation =
+        (Transformation<Object>)
+            applySinkProvider(
+                sinkTransform,
+                streamExecEnv,
+                runtimeProvider,
+                rowtimeFieldIndex,
+                sinkParallelism,
+                config,
+                classLoader);
+    return VeloxSourceSinkFactory.buildSink(
+        (Transformation) transformation,
+        Map.of(
+            Configuration.class.getName(),
+            streamExecEnv.getConfiguration(),
+            ResolvedSchema.class.getName(),
+            schema));
+    // --- End Gluten-specific code changes ---
   }
 
   /** Apply an operator to filter or report error to process not-null values for not-null fields. */
@@ -548,11 +556,6 @@ public abstract class CommonExecSink extends ExecNodeBase<Object>
     if (rowtimeFieldIndex == -1) {
       return inputTransform;
     }
-    // --- Begin Gluten-specific code changes ---
-    io.github.zhztheplayer.velox4j.type.RowType outputType =
-        (io.github.zhztheplayer.velox4j.type.RowType)
-            LogicalTypeConverter.toVLType(
-                ((InternalTypeInfo) inputTransform.getOutputType()).toLogicalType());
     return ExecNodeUtil.createOneInputTransformation(
         inputTransform,
         createTransformationMeta(
@@ -560,13 +563,10 @@ public abstract class CommonExecSink extends ExecNodeBase<Object>
             String.format("StreamRecordTimestampInserter(rowtime field: %s)", rowtimeFieldIndex),
             "StreamRecordTimestampInserter",
             config),
-        // TODO: support it, Map.of() will not be used, hardcode it here.
-        new GlutenOneInputOperator(
-            null, PlanNodeIdGenerator.newId(), null, Map.of("1", outputType)),
+        new StreamRecordTimestampInserter(rowtimeFieldIndex),
         inputTransform.getOutputType(),
         sinkParallelism,
         sinkParallelismConfigured);
-    // --- End Gluten-specific code changes ---
   }
 
   private InternalTypeInfo<RowData> getInputTypeInfo() {
