@@ -16,94 +16,59 @@
  */
 package org.apache.iceberg.spark.source
 
+import org.apache.gluten.ContentFileUtil
 import org.apache.gluten.backendsapi.BackendsApiManager
 import org.apache.gluten.exception.GlutenNotSupportException
+import org.apache.gluten.execution.SparkDataSourceRDDPartition
 import org.apache.gluten.substrait.rel.{IcebergLocalFilesBuilder, SplitInfo}
 import org.apache.gluten.substrait.rel.LocalFilesNode.ReadFileFormat
 
 import org.apache.spark.softaffinity.SoftAffinity
 import org.apache.spark.sql.catalyst.catalog.ExternalCatalogUtils
-import org.apache.spark.sql.connector.read.{InputPartition, Scan}
+import org.apache.spark.sql.connector.read.Scan
 import org.apache.spark.sql.types.StructType
 
 import org.apache.iceberg._
 import org.apache.iceberg.spark.SparkSchemaUtil
 
-import java.lang.{Long => JLong}
+import java.lang.{Class, Long => JLong}
 import java.util.{ArrayList => JArrayList, HashMap => JHashMap, List => JList, Map => JMap}
 
 import scala.collection.JavaConverters._
 
 object GlutenIcebergSourceUtil {
 
-  def genSplitInfoForPartition(
-      inputPartition: InputPartition,
-      index: Int,
-      readPartitionSchema: StructType): SplitInfo = inputPartition match {
-    case partition: SparkInputPartition =>
-      val paths = new JArrayList[String]()
-      val starts = new JArrayList[JLong]()
-      val lengths = new JArrayList[JLong]()
-      val partitionColumns = new JArrayList[JMap[String, String]]()
-      val deleteFilesList = new JArrayList[JList[DeleteFile]]()
-      var fileFormat = ReadFileFormat.UnknownFormat
+  def getClassOfSparkBatchQueryScan(): Class[SparkBatchQueryScan] = {
+    classOf[SparkBatchQueryScan]
+  }
 
-      val tasks = partition.taskGroup[ScanTask]().tasks().asScala
-      asFileScanTask(tasks.toList).foreach {
-        task =>
-          paths.add(
-            BackendsApiManager.getTransformerApiInstance
-              .encodeFilePathIfNeed(task.file().path().toString))
-          starts.add(task.start())
-          lengths.add(task.length())
-          partitionColumns.add(getPartitionColumns(task, readPartitionSchema))
-          deleteFilesList.add(task.deletes())
-          val currentFileFormat = convertFileFormat(task.file().format())
-          if (fileFormat == ReadFileFormat.UnknownFormat) {
-            fileFormat = currentFileFormat
-          } else if (fileFormat != currentFileFormat) {
-            throw new UnsupportedOperationException(
-              s"Only one file format is supported, " +
-                s"find different file format $fileFormat and $currentFileFormat")
-          }
-      }
-      val preferredLoc = SoftAffinity.getFilePartitionLocations(
-        paths.asScala.toArray,
-        inputPartition.preferredLocations())
-      IcebergLocalFilesBuilder.makeIcebergLocalFiles(
-        index,
-        paths,
-        starts,
-        lengths,
-        partitionColumns,
-        fileFormat,
-        preferredLoc.toList.asJava,
-        deleteFilesList
-      )
-    case _ =>
-      throw new UnsupportedOperationException("Only support iceberg SparkInputPartition.")
+  def deleteExists(p: SparkDataSourceRDDPartition): Boolean = {
+    p.inputPartitions.exists {
+      case ip: SparkInputPartition =>
+        val tasks = ip.taskGroup[ScanTask]().tasks().asScala
+        asFileScanTask(tasks.toList).exists(task => !task.deletes().isEmpty())
+      case _ => throw new UnsupportedOperationException(s"Unsupported InputPartition type")
+    }
   }
 
   def genSplitInfo(
-      inputPartitions: Seq[InputPartition],
-      index: Int,
+      partition: SparkDataSourceRDDPartition,
       readPartitionSchema: StructType): SplitInfo = {
     val paths = new JArrayList[String]()
     val starts = new JArrayList[JLong]()
     val lengths = new JArrayList[JLong]()
     val partitionColumns = new JArrayList[JMap[String, String]]()
     val deleteFilesList = new JArrayList[JList[DeleteFile]]()
-    val preferredLocs = new JArrayList[String]()
     var fileFormat = ReadFileFormat.UnknownFormat
 
-    inputPartitions.foreach {
+    partition.inputPartitions.foreach {
       case partition: SparkInputPartition =>
         val tasks = partition.taskGroup[ScanTask]().tasks().asScala
         asFileScanTask(tasks.toList).foreach {
           task =>
             paths.add(
               BackendsApiManager.getTransformerApiInstance
-                .encodeFilePathIfNeed(task.file().path().toString))
+                .encodeFilePathIfNeed(ContentFileUtil.getFilePath(task.file())))
             starts.add(task.start())
             lengths.add(task.length())
             partitionColumns.add(getPartitionColumns(task, readPartitionSchema))
@@ -117,17 +82,18 @@ object GlutenIcebergSourceUtil {
                   s"find different file format $fileFormat and $currentFileFormat")
             }
         }
-        preferredLocs.addAll(partition.preferredLocations().toList.asJava)
+      case o =>
+        throw new GlutenNotSupportException(s"Unsupported input partition type: $o")
     }
     IcebergLocalFilesBuilder.makeIcebergLocalFiles(
-      index,
+      partition.index,
       paths,
       starts,
       lengths,
       partitionColumns,
       fileFormat,
       SoftAffinity
-        .getFilePartitionLocations(paths.asScala.toArray, preferredLocs.asScala.toArray)
+        .getFilePartitionLocations(paths.asScala.toArray, partition.preferredLocations())
         .toList
         .asJava,
       deleteFilesList
