@@ -37,7 +37,8 @@ trait MetricsApi extends Serializable {
   def genInputIteratorTransformerMetrics(
       child: SparkPlan,
       sparkContext: SparkContext,
-      forBroadcast: Boolean): Map[String, SQLMetric]
+      forBroadcast: Boolean,
+      forShuffle: Boolean): Map[String, SQLMetric]
 
   def genInputIteratorTransformerMetricsUpdater(
       metrics: Map[String, SQLMetric],
@@ -103,6 +104,8 @@ trait MetricsApi extends Serializable {
 
   def genWriteFilesTransformerMetricsUpdater(metrics: Map[String, SQLMetric]): MetricsUpdater
 
+  def genBatchWriteMetrics(sparkContext: SparkContext): Map[String, SQLMetric]
+
   def genSortTransformerMetrics(sparkContext: SparkContext): Map[String, SQLMetric]
 
   def genSortTransformerMetricsUpdater(metrics: Map[String, SQLMetric]): MetricsUpdater
