@@ -27,7 +27,7 @@ import org.apache.gluten.substrait.expression.{ExpressionBuilder, ExpressionNode
 import org.apache.spark.ShuffleDependency
 import org.apache.spark.rdd.RDD
 import org.apache.spark.serializer.Serializer
-import org.apache.spark.shuffle.{GenShuffleWriterParameters, GlutenShuffleWriterWrapper}
+import org.apache.spark.shuffle.{GenShuffleReaderParameters, GenShuffleWriterParameters, GlutenShuffleReaderWrapper, GlutenShuffleWriterWrapper}
 import org.apache.spark.sql.catalyst.catalog.BucketSpec
 import org.apache.spark.sql.catalyst.catalog.CatalogTypes.TablePartitionSpec
 import org.apache.spark.sql.catalyst.expressions._
@@ -38,7 +38,6 @@ import org.apache.spark.sql.catalyst.plans.JoinType
 import org.apache.spark.sql.catalyst.plans.physical.{BroadcastMode, Partitioning}
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.datasources.FileFormat
-import org.apache.spark.sql.execution.datasources.v2.FileScan
 import org.apache.spark.sql.execution.exchange.ShuffleExchangeExec
 import org.apache.spark.sql.execution.joins.BuildSideRelation
 import org.apache.spark.sql.execution.metric.SQLMetric
@@ -242,6 +241,8 @@ trait SparkPlanExecApi {
     GenericExpressionTransformer(substraitExprName, Seq(left, right), original)
   }
 
+  def getDecimalArithmeticExprName(exprName: String): String = exprName
+
   /** Transform map_entries to Substrait. */
   def genMapEntriesTransformer(
       substraitExprName: String,
@@ -385,6 +386,9 @@ trait SparkPlanExecApi {
   def genColumnarShuffleWriter[K, V](
       parameters: GenShuffleWriterParameters[K, V]): GlutenShuffleWriterWrapper[K, V]
 
+  def genColumnarShuffleReader[K, C](
+      parameters: GenShuffleReaderParameters[K, C]): GlutenShuffleReaderWrapper[K, C]
+
   /**
    * Generate ColumnarBatchSerializer for ColumnarShuffleExchangeExec.
    *
@@ -470,6 +474,12 @@ trait SparkPlanExecApi {
     TruncTimestampTransformer(substraitExprName, format, timestamp, original)
   }
 
+  def genToUnixTimestampTransformer(
+      substraitExprName: String,
+      timeExp: ExpressionTransformer,
+      format: ExpressionTransformer,
+      original: Expression): ExpressionTransformer
+
   def genDateDiffTransformer(
       substraitExprName: String,
       endDate: ExpressionTransformer,
@@ -641,41 +651,15 @@ trait SparkPlanExecApi {
 
   def rewriteSpillPath(path: String): String = path
 
-  /**
-   * Vanilla spark just push down part of filter condition into scan, however gluten can push down
-   * all filters. This function calculates the remaining conditions in FilterExec, add into the
-   * dataFilters of the leaf node.
-   * @param extraFilters:
-   *   Conjunctive Predicates, which are split from the upper FilterExec
-   * @param sparkExecNode:
-   *   The vanilla leaf node of the plan tree, which is FileSourceScanExec or BatchScanExec
-   * @return
-   *   return all push down filters
-   */
-  def postProcessPushDownFilter(
-      extraFilters: Seq[Expression],
-      sparkExecNode: LeafExecNode): Seq[Expression] = {
-    def getPushedFilter(dataFilters: Seq[Expression]): Seq[Expression] = {
-      val pushedFilters =
-        dataFilters ++ FilterHandler.getRemainingFilters(dataFilters, extraFilters)
-      pushedFilters.filterNot(_.references.exists {
-        attr => BackendsApiManager.getSparkPlanExecApiInstance.isRowIndexMetadataColumn(attr.name)
-      })
-    }
-    sparkExecNode match {
-      case fileSourceScan: FileSourceScanExecTransformerBase =>
-        getPushedFilter(fileSourceScan.dataFilters)
-      case batchScan: BatchScanExecTransformerBase =>
-        batchScan.scan match {
-          case fileScan: FileScan =>
-            getPushedFilter(fileScan.dataFilters)
-          case _ =>
-            // TODO: For data lake format use pushedFilters in SupportsPushDownFilters
-            extraFilters
-        }
-      case _ =>
-        throw new GlutenNotSupportException(s"${sparkExecNode.getClass.toString} is not supported.")
-    }
+  def supportPushDownFilterToScan(sparkExecNode: LeafExecNode): Boolean = true
+
+  /** Return whether the filter is supported in scan. */
+  def isSupportedScanFilter(filter: Expression, sparkExecNode: LeafExecNode): Boolean = {
+    ExpressionConverter.canReplaceWithExpressionTransformer(
+      ExpressionConverter.replaceAttributeReference(filter),
+      sparkExecNode.output) &&
+    (!filter.references.exists(
+      attr => BackendsApiManager.getSparkPlanExecApiInstance.isRowIndexMetadataColumn(attr.name)))
   }
 
   def genGenerateTransformer(
@@ -719,7 +703,7 @@ trait SparkPlanExecApi {
       orderSpec: Seq[SortOrder],
       rankLikeFunction: Expression,
       limit: Int,
-      mode: WindowGroupLimitMode,
+      mode: GlutenWindowGroupLimitMode,
       child: SparkPlan): SparkPlan =
     WindowGroupLimitExecTransformer(partitionSpec, orderSpec, rankLikeFunction, limit, mode, child)
 
@@ -742,14 +726,7 @@ trait SparkPlanExecApi {
       plan: SparkPlan,
       offset: Int): ColumnarCollectLimitBaseExec
 
-  def genColumnarRangeExec(
-      start: Long,
-      end: Long,
-      step: Long,
-      numSlices: Int,
-      numElements: BigInt,
-      outputAttributes: Seq[Attribute],
-      child: Seq[SparkPlan]): ColumnarRangeBaseExec
+  def genColumnarRangeExec(rangeExec: RangeExec): ColumnarRangeBaseExec
 
   def genColumnarTailExec(limit: Int, plan: SparkPlan): ColumnarCollectTailBaseExec
 
@@ -791,7 +768,18 @@ trait SparkPlanExecApi {
     throw new GlutenNotSupportException("timestampdiff is not supported")
   }
 
+  def genMonthsBetweenTransformer(
+      substraitExprName: String,
+      date1: ExpressionTransformer,
+      date2: ExpressionTransformer,
+      roundOff: ExpressionTransformer,
+      original: MonthsBetween): ExpressionTransformer
+
   def isRowIndexMetadataColumn(columnName: String): Boolean = {
     SparkShimLoader.getSparkShims.isRowIndexMetadataColumn(columnName)
   }
+
+  def getErrorMessage(raiseError: RaiseError): Expression = {
+    throw new GlutenNotSupportException(s"${ExpressionNames.RAISE_ERROR} is not supported")
+  }
 }
