@@ -137,12 +137,16 @@ case class TakeOrderedAndProjectExecTransformer(
           LimitExecTransformer(localSortPlan, limitBeforeShuffleOffset, limit)
       }
       val transformStageCounter: AtomicInteger =
-        ColumnarCollapseTransformStages.transformStageCounter
+        ColumnarCollapseTransformStages.getTransformStageCounter(child)
       val finalLimitPlan = if (hasShuffle) {
         limitBeforeShuffle
       } else {
         val limitStagePlan =
           WholeStageTransformer(limitBeforeShuffle)(transformStageCounter.incrementAndGet())
+        val cudfTag = child.getTagValue(CudfTag.CudfTag)
+        if (cudfTag.isDefined) {
+          limitStagePlan.setTagValue(CudfTag.CudfTag, cudfTag.get)
+        }
         val shuffleExec = ShuffleExchangeExec(SinglePartition, limitStagePlan)
         val transformedShuffleExec =
           ColumnarShuffleExchangeExec(shuffleExec, limitStagePlan, shuffleExec.child.output)
@@ -166,7 +170,7 @@ case class TakeOrderedAndProjectExecTransformer(
 
       val finalPlan =
         WholeStageTransformer(collapsed)(transformStageCounter.incrementAndGet())
-
+      finalPlan.copyTagsFrom(child)
       finalPlan.executeColumnar()
     }
   }
