@@ -17,8 +17,7 @@
 package org.apache.gluten.execution
 
 import org.apache.gluten.backendsapi.BackendsApiManager
-import org.apache.gluten.config.{GlutenConfig, GlutenNumaBindingInfo}
-import org.apache.gluten.exception.GlutenException
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.expression._
 import org.apache.gluten.extension.columnar.transition.Convention
 import org.apache.gluten.metrics.{GlutenTimeMetric, MetricsUpdater}
@@ -26,6 +25,7 @@ import org.apache.gluten.substrait.`type`.{TypeBuilder, TypeNode}
 import org.apache.gluten.substrait.SubstraitContext
 import org.apache.gluten.substrait.plan.{PlanBuilder, PlanNode}
 import org.apache.gluten.substrait.rel.{LocalFilesNode, RelNode, SplitInfo}
+import org.apache.gluten.substrait.rel.LocalFilesNode.ReadFileFormat
 import org.apache.gluten.utils.SubstraitPlanPrinterUtil
 
 import org.apache.spark._
@@ -35,7 +35,6 @@ import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{Attribute, SortOrder}
 import org.apache.spark.sql.catalyst.plans.physical.Partitioning
 import org.apache.spark.sql.catalyst.trees.TreeNodeTag
-import org.apache.spark.sql.connector.read.InputPartition
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.metric.SQLMetric
 import org.apache.spark.sql.utils.SparkInputMetricsUtil.InputMetricsWrapper
@@ -46,7 +45,6 @@ import org.apache.hadoop.fs.viewfs.ViewFileSystemUtils
 
 import scala.collection.JavaConverters._
 import scala.collection.mutable
-import scala.collection.mutable.ArrayBuffer
 
 case class TransformContext(outputAttributes: Seq[Attribute], root: RelNode)
 
@@ -71,7 +69,7 @@ trait TransformSupport extends ValidatablePlan {
       s"${this.getClass.getSimpleName} doesn't support doExecute")
   }
 
-  protected def isCudf: Boolean = getTagValue[Boolean](CudfTag.CudfTag).getOrElse(false)
+  def isCudf: Boolean = getTagValue[Boolean](CudfTag.CudfTag).getOrElse(false)
 
   // Use super.nodeName will cause exception scala 213 Super calls can only target methods
   // for FileSourceScan.
@@ -136,10 +134,18 @@ trait TransformSupport extends ValidatablePlan {
 
 trait LeafTransformSupport extends TransformSupport with LeafExecNode {
   final override def columnarInputRDDs: Seq[RDD[ColumnarBatch]] = Seq.empty
+
+  /** Returns the split infos that will be processed by the underlying native engine. */
   def getSplitInfos: Seq[SplitInfo]
 
   /** Returns the partitions generated by this data source scan. */
-  def getPartitions: Seq[InputPartition]
+  def getPartitions: Seq[Partition]
+
+  /** Returns the partitions generated by this data source scan and tied with ReadFileFormat. */
+  def getPartitionWithReadFileFormats: Seq[(Partition, ReadFileFormat)]
+
+  /** Returns distinct partition's ReadFileFormats. */
+  def getDistinctPartitionReadFileFormats: Set[ReadFileFormat] = Set.empty
 }
 
 trait UnaryTransformSupport extends TransformSupport with UnaryExecNode {
@@ -149,7 +155,7 @@ trait UnaryTransformSupport extends TransformSupport with UnaryExecNode {
 }
 
 case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = false)(
-    val transformStageId: Int
+    var transformStageId: Int
 ) extends WholeStageTransformerGenerateTreeStringShim
   with UnaryTransformSupport {
 
@@ -163,10 +169,6 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
   @transient override lazy val metrics: Map[String, SQLMetric] =
     BackendsApiManager.getMetricsApiInstance.genWholeStageTransformerMetrics(sparkContext)
 
-  val sparkConf: SparkConf = sparkContext.getConf
-
-  val numaBindingInfo: GlutenNumaBindingInfo = GlutenConfig.get.numaBindingInfo
-
   @transient
   private var wholeStageTransformerContext: Option[WholeStageTransformContext] = None
 
@@ -236,20 +238,14 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
       .asInstanceOf[TransformSupport]
       .transform(substraitContext)
     if (childCtx == null) {
-      throw new NullPointerException(s"WholeStageTransformer can't do Transform on $child")
+      throw new IllegalStateException(s"WholeStageTransformer can't do Transform on $child")
     }
 
-    val outNames = new java.util.ArrayList[String]()
-    for (attr <- childCtx.outputAttributes) {
-      outNames.add(ConverterUtils.genColumnNameWithExprId(attr))
-    }
+    val outNames = childCtx.outputAttributes.map(ConverterUtils.genColumnNameWithExprId).asJava
 
     val planNode = if (BackendsApiManager.getSettings.needOutputSchemaForPlan()) {
-      val outputSchema = if (outputSchemaForPlan.isDefined) {
-        outputSchemaForPlan.get
-      } else {
-        inferSchemaFromAttributes(childCtx.outputAttributes)
-      }
+      val outputSchema =
+        outputSchemaForPlan.getOrElse(inferSchemaFromAttributes(childCtx.outputAttributes))
 
       PlanBuilder.makePlan(
         substraitContext,
@@ -274,61 +270,53 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
 
   /** Find all [[LeafTransformSupport]] in one WholeStageTransformer */
   private def findAllLeafTransformers(): Seq[LeafTransformSupport] = {
-    val allLeafTransformers = new mutable.ListBuffer[LeafTransformSupport]()
 
-    def transformChildren(
-        plan: SparkPlan,
-        leafTransformers: mutable.ListBuffer[LeafTransformSupport]): Unit = {
-      if (plan != null && plan.isInstanceOf[TransformSupport]) {
-        plan match {
-          case transformer: LeafTransformSupport =>
-            leafTransformers.append(transformer)
-          case _ =>
-        }
-
-        // according to the substrait plan order
-        // SHJ may include two leaves in a whole stage.
-        plan match {
-          case shj: HashJoinLikeExecTransformer =>
-            transformChildren(shj.streamedPlan, leafTransformers)
-            transformChildren(shj.buildPlan, leafTransformers)
-          case t: TransformSupport =>
-            t.children
-              .foreach(transformChildren(_, leafTransformers))
-        }
-      }
+    def collectLeafTransformers(plan: SparkPlan): Seq[LeafTransformSupport] = plan match {
+      case transformer: LeafTransformSupport =>
+        Seq(transformer)
+      case shj: HashJoinLikeExecTransformer =>
+        collectLeafTransformers(shj.streamedPlan) ++ collectLeafTransformers(shj.buildPlan)
+      case t: TransformSupport =>
+        t.children.flatMap(collectLeafTransformers)
+      case _ =>
+        Seq.empty
     }
 
-    transformChildren(child, allLeafTransformers)
-    allLeafTransformers.toSeq
+    collectLeafTransformers(child)
   }
 
+  /**
+   * If containing leaf exec transformer this "whole stage" generates a RDD which itself takes care
+   * of [[LeafTransformSupport]] there won't be any other RDD for leaf operator. As a result,
+   * genFirstStageIterator rather than genFinalStageIterator will be invoked
+   */
   private def generateWholeStageRDD(
       leafTransformers: Seq[LeafTransformSupport],
       wsCtx: WholeStageTransformContext,
       inputRDDs: ColumnarInputRDDsWrapper,
       pipelineTime: SQLMetric): RDD[ColumnarBatch] = {
-    val isKeyGroupPartition = leafTransformers.exists {
-      // TODO: May can apply to BatchScanExecTransformer without key group partitioning
-      case b: BatchScanExecTransformerBase if b.keyGroupedPartitioning.isDefined => true
-      case _ => false
-    }
 
-    /**
-     * If containing leaf exec transformer this "whole stage" generates a RDD which itself takes
-     * care of [[LeafTransformSupport]] there won't be any other RDD for leaf operator. As a result,
-     * genFirstStageIterator rather than genFinalStageIterator will be invoked
-     */
-    val allInputPartitions = leafTransformers.map(
-      leafTransformer => {
-        if (isKeyGroupPartition) {
-          leafTransformer.asInstanceOf[BatchScanExecTransformerBase].getPartitionsWithIndex
-        } else {
-          Seq(leafTransformer.getPartitions)
-        }
-      })
-
-    val allSplitInfos = getSplitInfosFromPartitions(isKeyGroupPartition, leafTransformers)
+    // If these are two leaf transformers, they must have same partitions,
+    // otherwise, exchange will be inserted. We should combine the two leaf
+    // transformers' partitions with same index, and set them together in
+    // the substraitContext.
+    // We use transpose to do that, You can refer to
+    // the diagram below.
+    // leaf1  p11 p12 p13 p14 ... p1n
+    // leaf2  p21 p22 p23 p24 ... p2n
+    // transpose =>
+    // leaf1 | leaf2
+    //  p11  |  p21    => substraitContext.setSplitInfo([p11, p21])
+    //  p12  |  p22    => substraitContext.setSplitInfo([p12, p22])
+    //  p13  |  p23    ...
+    //  p14  |  p24
+    //      ...
+    //  p1n  |  p2n    => substraitContext.setSplitInfo([p1n, p2n])
+    // The data in partition may be empty, for example,
+    // if these are two batch scan transformer with keyGroupPartitioning,
+    // they have same partitionValues,
+    // but some partitions maybe empty for hose partition values that are not present.
+    val allSplitInfos = leafTransformers.map(_.getSplitInfos).transpose
 
     if (GlutenConfig.get.enableHdfsViewfs) {
       val viewfsToHdfsCache: mutable.Map[String, String] = mutable.Map.empty
@@ -366,6 +354,7 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
       wsCtx.enableCudf
     )
 
+    val allInputPartitions = leafTransformers.map(_.getPartitions)
     SoftAffinity.updateFilePartitionLocations(allInputPartitions, rdd.id)
 
     leafTransformers.foreach {
@@ -377,55 +366,6 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
     rdd
   }
 
-  private def getSplitInfosFromPartitions(
-      isKeyGroupPartition: Boolean,
-      leafTransformers: Seq[LeafTransformSupport]): Seq[Seq[SplitInfo]] = {
-    val allSplitInfos = if (isKeyGroupPartition) {
-      // If these are two batch scan transformer with keyGroupPartitioning,
-      // they have same partitionValues,
-      // but some partitions maybe empty for those partition values that are not present,
-      // otherwise, exchange will be inserted. We should combine the two leaf
-      // transformers' partitions with same index, and set them together in
-      // the substraitContext. We use transpose to do that, You can refer to
-      // the diagram below.
-      // leaf1  Seq(p11) Seq(p12, p13) Seq(p14) ... Seq(p1n)
-      // leaf2  Seq(p21) Seq(p22)      Seq()    ... Seq(p2n)
-      // transpose =>
-      // leaf1 | leaf2
-      //  Seq(p11)       |  Seq(p21)    => substraitContext.setSplitInfo([Seq(p11), Seq(p21)])
-      //  Seq(p12, p13)  |  Seq(p22)    => substraitContext.setSplitInfo([Seq(p12, p13), Seq(p22)])
-      //  Seq(p14)       |  Seq()    ...
-      //                ...
-      //  Seq(p1n)       |  Seq(p2n)    => substraitContext.setSplitInfo([Seq(p1n), Seq(p2n)])
-      leafTransformers.map(_.asInstanceOf[BatchScanExecTransformerBase].getSplitInfosWithIndex)
-    } else {
-      // If these are two leaf transformers, they must have same partitions,
-      // otherwise, exchange will be inserted. We should combine the two leaf
-      // transformers' partitions with same index, and set them together in
-      // the substraitContext. We use transpose to do that, You can refer to
-      // the diagram below.
-      // leaf1  p11 p12 p13 p14 ... p1n
-      // leaf2  p21 p22 p23 p24 ... p2n
-      // transpose =>
-      // leaf1 | leaf2
-      //  p11  |  p21    => substraitContext.setSplitInfo([p11, p21])
-      //  p12  |  p22    => substraitContext.setSplitInfo([p12, p22])
-      //  p13  |  p23    ...
-      //  p14  |  p24
-      //      ...
-      //  p1n  |  p2n    => substraitContext.setSplitInfo([p1n, p2n])
-      leafTransformers.map(_.getSplitInfos)
-    }
-
-    val partitionLength = allSplitInfos.head.size
-    if (allSplitInfos.exists(_.size != partitionLength)) {
-      throw new GlutenException(
-        "The partition length of all the leaf transformer are not the same.")
-    }
-
-    allSplitInfos.transpose
-  }
-
   override def doExecuteColumnar(): RDD[ColumnarBatch] = {
     assert(child.isInstanceOf[TransformSupport])
     val pipelineTime: SQLMetric = longMetric("pipelineTime")
@@ -445,17 +385,19 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
     } else {
 
       /**
-       * the whole stage contains NO [[LeafTransformSupport]]. this the default case for:
-       *   1. SCAN with clickhouse backend (check ColumnarCollapseTransformStages#separateScanRDD())
-       *      2. test case where query plan is constructed from simple dataframes (e.g.
-       *      GlutenDataFrameAggregateSuite) in these cases, separate RDDs takes care of SCAN as a
-       *      result, genFinalStageIterator rather than genFirstStageIterator will be invoked
+       * the whole stage contains NO [[LeafTransformSupport]]. This is the default case for:
+       *   - SCAN of clickhouse backend. See
+       *     BackendsApiManager.getSettings.excludeScanExecFromCollapsedStage.
+       *   - Test case where query plan is constructed from simple DataFrames, e.g.
+       *     GlutenDataFrameAggregateSuite.
+       *
+       * In these cases, separate RDDs take care of SCAN. As a result, genFinalStageIterator rather
+       * than genFirstStageIterator will be invoked.
        */
       new WholeStageZippedPartitionsRDD(
         sparkContext,
         inputRDDs,
-        numaBindingInfo,
-        sparkConf,
+        sparkContext.getConf,
         wsCtx,
         pipelineTime,
         BackendsApiManager.getMetricsApiInstance.metricsUpdatingFunction(
@@ -477,20 +419,11 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
   }
 
   private def leafInputMetricsUpdater(): InputMetricsWrapper => Unit = {
-    def collectLeaves(plan: SparkPlan, buffer: ArrayBuffer[TransformSupport]): Unit = {
-      plan match {
-        case node: TransformSupport if node.children.forall(!_.isInstanceOf[TransformSupport]) =>
-          buffer.append(node)
-        case node: TransformSupport =>
-          node.children
-            .foreach(collectLeaves(_, buffer))
-        case _ =>
-      }
+    val leaves = child.collect {
+      case plan: TransformSupport if plan.children.forall(!_.isInstanceOf[TransformSupport]) =>
+        plan
     }
-
-    val leafBuffer = new ArrayBuffer[TransformSupport]()
-    collectLeaves(child, leafBuffer)
-    val leafMetricsUpdater = leafBuffer.map(_.metricsUpdater())
+    val leafMetricsUpdater = leaves.map(_.metricsUpdater())
 
     (inputMetrics: InputMetricsWrapper) => {
       leafMetricsUpdater.foreach(_.updateInputMetrics(inputMetrics))
