@@ -89,7 +89,7 @@ object NativeWriteFilesWithSkippingSortAndProject extends Logging {
     def allSortOrdersFromPartitionColumns(sortOrders: Seq[SortOrder]): Boolean = {
       val partitionColumnsSet = AttributeSet(partitionColumns)
       sortOrders.forall(_.direction == Ascending) &&
-      sortOrders.size == partitionColumnsSet.size &&
+      sortOrders.size <= partitionColumnsSet.size &&
       sortOrders.map(_.references).forall(attrs => attrs.subsetOf(partitionColumnsSet))
     }
 
