@@ -25,8 +25,10 @@ import org.apache.gluten.substrait.expression.ExpressionNode
 import org.apache.spark.sql.catalyst.expressions.{Attribute, Expression}
 import org.apache.spark.sql.catalyst.plans.{ExistenceJoin, FullOuter, InnerLike, JoinType, LeftAnti, LeftOuter, LeftSemi, RightOuter}
 
+import com.google.protobuf.{Any, DoubleValue, Int32Value, Int64Value, Message, StringValue}
 import io.substrait.proto.{CrossRel, JoinRel, NamedStruct, Type}
 
+import java.lang.{Double => JDouble, Long => JLong}
 import java.util.{Collections, List => JList}
 
 import scala.collection.JavaConverters._
@@ -107,4 +109,18 @@ object SubstraitUtil {
     val nameList = ConverterUtils.collectAttributeNamesWithExprId(output)
     createNameStructBuilder(typeList, nameList, Collections.emptyList()).build()
   }
+
+  def convertJavaObjectToAny(obj: AnyRef): Any = {
+    if (obj == null) return null
+    val msg: Message = obj match {
+      case s: String => StringValue.newBuilder.setValue(s).build
+      case i: Integer => Int32Value.newBuilder.setValue(i).build
+      case l: JLong => Int64Value.newBuilder.setValue(l).build
+      case d: JDouble => DoubleValue.newBuilder.setValue(d).build
+      case _ =>
+        // TODO: generate the message according to the object type
+        StringValue.newBuilder.setValue(obj.toString).build
+    }
+    BackendsApiManager.getTransformerApiInstance.packPBMessage(msg)
+  }
 }
