@@ -31,8 +31,8 @@ import org.apache.spark.sql.catalyst.expressions.{Attribute, SortOrder}
 import org.apache.spark.sql.catalyst.plans.physical.Partitioning
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.catalyst.util.truncatedString
-import org.apache.spark.sql.execution.adaptive.BroadcastQueryStageExec
-import org.apache.spark.sql.execution.exchange.BroadcastExchangeLike
+import org.apache.spark.sql.execution.adaptive.{BroadcastQueryStageExec, ShuffleQueryStageExec}
+import org.apache.spark.sql.execution.exchange.{BroadcastExchangeLike, ShuffleExchangeLike}
 import org.apache.spark.sql.execution.metric.SQLMetric
 import org.apache.spark.sql.vectorized.ColumnarBatch
 
@@ -52,7 +52,8 @@ case class InputIteratorTransformer(child: SparkPlan) extends UnaryTransformSupp
     BackendsApiManager.getMetricsApiInstance.genInputIteratorTransformerMetrics(
       child,
       sparkContext,
-      forBroadcast())
+      forBroadcast(),
+      forShuffle())
 
   override def simpleString(maxFields: Int): String = {
     s"$nodeName${truncatedString(output, "[", ", ", "]", maxFields)}"
@@ -84,8 +85,19 @@ case class InputIteratorTransformer(child: SparkPlan) extends UnaryTransformSupp
 
   private def forBroadcast(): Boolean = {
     child match {
-      case ColumnarInputAdapter(c) if c.isInstanceOf[BroadcastQueryStageExec] => true
-      case ColumnarInputAdapter(c) if c.isInstanceOf[BroadcastExchangeLike] => true
+      case ColumnarInputAdapter(c)
+          if c.isInstanceOf[BroadcastQueryStageExec] ||
+            c.isInstanceOf[BroadcastExchangeLike] =>
+        true
+      case _ => false
+    }
+  }
+
+  private def forShuffle(): Boolean = {
+    child match {
+      case ColumnarInputAdapter(c)
+          if c.isInstanceOf[ShuffleQueryStageExec] || c.isInstanceOf[ShuffleExchangeLike] =>
+        true
       case _ => false
     }
   }
@@ -127,13 +139,7 @@ case class InputIteratorTransformer(child: SparkPlan) extends UnaryTransformSupp
  * created, e.g. for special fallback handling when an existing WholeStageTransformer failed to
  * generate/compile code.
  */
-case class ColumnarCollapseTransformStages(
-    glutenConf: GlutenConfig,
-    transformStageCounter: AtomicInteger = ColumnarCollapseTransformStages.transformStageCounter)
-  extends Rule[SparkPlan] {
-
-  def separateScanRDD: Boolean =
-    BackendsApiManager.getSettings.excludeScanExecFromCollapsedStage()
+case class ColumnarCollapseTransformStages(glutenConf: GlutenConfig) extends Rule[SparkPlan] {
 
   def apply(plan: SparkPlan): SparkPlan = {
     insertWholeStageTransformer(plan)
@@ -144,7 +150,8 @@ case class ColumnarCollapseTransformStages(
    * WholeStageTransformer.
    */
   private def isSeparateBaseScanExecTransformer(plan: SparkPlan): Boolean = plan match {
-    case _: BasicScanExecTransformer if separateScanRDD => true
+    case _: BasicScanExecTransformer =>
+      BackendsApiManager.getSettings.excludeScanExecFromCollapsedStage()
     case _ => false
   }
 
@@ -156,7 +163,9 @@ case class ColumnarCollapseTransformStages(
   /** Inserts an InputIteratorTransformer on top of those that do not support transform. */
   private def insertInputIteratorTransformer(plan: SparkPlan): SparkPlan = {
     plan match {
-      case p if !supportTransform(p) =>
+      case p if p.isInstanceOf[WholeStageTransformer] || !supportTransform(p) =>
+        // TODO: if p.isInstanceOf[WholeStageTransformer], we can merge two whole stage
+        //  transformers.
         ColumnarCollapseTransformStages.wrapInputIteratorTransformer(insertWholeStageTransformer(p))
       case p =>
         p.withNewChildren(p.children.map(insertInputIteratorTransformer))
@@ -165,9 +174,10 @@ case class ColumnarCollapseTransformStages(
 
   private def insertWholeStageTransformer(plan: SparkPlan): SparkPlan = {
     plan match {
+      case wst: WholeStageTransformer => wst
       case t if supportTransform(t) =>
-        WholeStageTransformer(t.withNewChildren(t.children.map(insertInputIteratorTransformer)))(
-          transformStageCounter.incrementAndGet())
+        // transformStageId will be updated by rule `GenerateTransformStageId`.
+        WholeStageTransformer(t.withNewChildren(t.children.map(insertInputIteratorTransformer)))(-1)
       case other =>
         other.withNewChildren(other.children.map(insertWholeStageTransformer))
     }
@@ -203,9 +213,20 @@ case class ColumnarInputAdapter(child: SparkPlan)
 }
 
 object ColumnarCollapseTransformStages {
-  val transformStageCounter = new AtomicInteger(0)
-
   def wrapInputIteratorTransformer(plan: SparkPlan): TransformSupport = {
     InputIteratorTransformer(ColumnarInputAdapter(plan))
   }
+
+  def getTransformStageCounter(plan: SparkPlan): AtomicInteger = {
+    new AtomicInteger(findMaxTransformStageId(plan))
+  }
+
+  private def findMaxTransformStageId(plan: SparkPlan): Int = {
+    plan match {
+      case wst: WholeStageTransformer =>
+        wst.transformStageId
+      case _ =>
+        plan.children.map(findMaxTransformStageId).foldLeft(0)(Math.max)
+    }
+  }
 }
