@@ -23,7 +23,8 @@ import org.apache.spark.SparkConf
 import org.apache.spark.sql.Row
 import org.apache.spark.sql.catalyst.TableIdentifier
 import org.apache.spark.sql.execution.SparkPlan
-import org.apache.spark.sql.hive.HiveTableScanExecTransformer
+import org.apache.spark.sql.hive.{HiveExternalCatalog, HiveTableScanExecTransformer}
+import org.apache.spark.sql.hive.client.HiveClient
 
 class GlutenHiveSQLQuerySuite extends GlutenHiveSQLQuerySuiteBase {
 
@@ -136,4 +137,34 @@ class GlutenHiveSQLQuerySuite extends GlutenHiveSQLQuerySuiteBase {
       purge = false)
   }
 
+  test("GLUTEN-11062: Supports mixed input format for partitioned Hive table") {
+    val hiveClient: HiveClient =
+      spark.sharedState.externalCatalog.unwrapped.asInstanceOf[HiveExternalCatalog].client
+
+    withSQLConf("spark.sql.hive.convertMetastoreParquet" -> "false") {
+      withTempDir {
+        dir =>
+          val parquetLoc = s"file:///$dir/test_parquet"
+          val orcLoc = s"file:///$dir/test_orc"
+          withTable("test_parquet", "test_orc") {
+            hiveClient.runSqlHive(s"""create table test_parquet(id int)
+                 partitioned by(pid int)
+                 stored as parquet location '$parquetLoc'
+                 """.stripMargin)
+            hiveClient.runSqlHive("insert into test_parquet partition(pid=1) select 2")
+            hiveClient.runSqlHive(s"""create table test_orc(id int)
+                 partitioned by(pid int)
+                 stored as orc location '$orcLoc'
+                 """.stripMargin)
+            hiveClient.runSqlHive("insert into test_orc partition(pid=2) select 2")
+            hiveClient.runSqlHive(
+              s"alter table test_parquet add partition (pid=2) location '$orcLoc/pid=2'")
+            hiveClient.runSqlHive("alter table test_parquet partition(pid=2) SET FILEFORMAT orc")
+            val df = sql("select pid, id from test_parquet order by pid")
+            checkAnswer(df, Seq(Row(1, 2), Row(2, 2)))
+            checkOperatorMatch[HiveTableScanExecTransformer](df)
+          }
+      }
+    }
+  }
 }
