@@ -23,6 +23,7 @@ import org.apache.gluten.execution.FileSourceScanExecTransformer
 import org.apache.gluten.utils.BackendTestUtils
 
 import org.apache.spark.SparkConf
+import org.apache.spark.internal.config.UI.UI_ENABLED
 import org.apache.spark.scheduler.{SparkListener, SparkListenerEvent}
 import org.apache.spark.sql.{GlutenSQLTestsTrait, Row}
 import org.apache.spark.sql.execution.ProjectExec
@@ -37,6 +38,9 @@ class GlutenFallbackSuite extends GlutenSQLTestsTrait with AdaptiveSparkPlanHelp
   override def sparkConf: SparkConf = {
     super.sparkConf
       .set(GlutenConfig.RAS_ENABLED.key, "false")
+      .set(GlutenConfig.GLUTEN_UI_ENABLED.key, "true")
+      // The gluten ui event test suite expects the spark ui to be enable
+      .set(UI_ENABLED, true)
   }
 
   testGluten("test fallback logging") {
@@ -53,7 +57,7 @@ class GlutenFallbackSuite extends GlutenSQLTestsTrait with AdaptiveSparkPlanHelp
       val msgRegex =
         """Validation failed for plan: Scan parquet spark_catalog.default\.t\[QueryId=[0-9]+\],""" +
           """ due to: \[FallbackByUserOptions\] Validation failed on node Scan parquet""" +
-          """ spark_catalog\.default\.t\.""".stripMargin
+          """ spark_catalog\.default\.t""".stripMargin
       assert(testAppender.loggingEvents.exists(_.getMessage.getFormattedMessage.matches(msgRegex)))
     }
   }
