@@ -16,17 +16,15 @@
  */
 package org.apache.gluten.integration.command;
 
+import org.apache.gluten.integration.QuerySet;
 import org.apache.gluten.integration.Suite;
 import org.apache.gluten.integration.action.Actions;
-import org.apache.gluten.utils.CollectionConverter;
+import org.apache.gluten.integration.collections.JavaCollectionConverter;
 
 import com.google.common.base.Preconditions;
 import picocli.CommandLine;
 
 import java.util.*;
-import java.util.stream.Collectors;
-
-import scala.collection.immutable.Seq;
 
 public class QueriesMixin {
   @CommandLine.Option(
@@ -93,63 +91,25 @@ public class QueriesMixin {
   public Actions.QuerySelector queries() {
     return new Actions.QuerySelector() {
       @Override
-      public Seq<String> select(Suite suite) {
-        final List<String> all = select0(suite);
-        final Division div = Division.parse(shard);
-        final List<String> out = div(all, div);
-        System.out.println("About to run queries: " + out + "... ");
-        return CollectionConverter.toImmutable(out);
-      }
-
-      private List<String> div(List<String> from, Division div) {
-        final int queryCount = from.size();
-        final int shardCount = div.shardCount;
-        final int least = queryCount / shardCount;
-        final int shardIdx = div.shard - 1;
-        final int shardStart = shardIdx * least;
-        final int numQueriesInShard;
-        if (shardIdx == shardCount - 1) {
-          final int remaining = queryCount - least * shardCount;
-          numQueriesInShard = least + remaining;
-        } else {
-          numQueriesInShard = least;
-        }
-        final List<String> out = new ArrayList<>();
-        for (int i = shardStart; i < shardStart + numQueriesInShard; i++) {
-          out.add(from.get(i));
-        }
-        return out;
-      }
-
-      private List<String> select0(Suite suite) {
+      public QuerySet select(Suite suite) {
         final String[] queryIds = queries;
         final String[] excludedQueryIds = excludedQueries;
         if (queryIds.length > 0 && excludedQueryIds.length > 0) {
           throw new IllegalArgumentException(
               "Should not specify queries and excluded queries at the same time");
         }
-        String[] all = suite.allQueryIds();
-        Set<String> allSet = new HashSet<>(Arrays.asList(all));
+        QuerySet querySet = suite.allQueries();
         if (queryIds.length > 0) {
-          for (String id : queryIds) {
-            if (!allSet.contains(id)) {
-              throw new IllegalArgumentException("Invalid query ID: " + id);
-            }
-          }
-          return Arrays.asList(queryIds);
+          querySet = querySet.filter(JavaCollectionConverter.asScalaSeq(Arrays.asList(queryIds)));
         }
         if (excludedQueryIds.length > 0) {
-          for (String id : excludedQueryIds) {
-            if (!allSet.contains(id)) {
-              throw new IllegalArgumentException("Invalid query ID to exclude: " + id);
-            }
-          }
-          Set<String> excludedSet = new HashSet<>(Arrays.asList(excludedQueryIds));
-          return Arrays.stream(all)
-              .filter(id -> !excludedSet.contains(id))
-              .collect(Collectors.toList());
+          querySet =
+              querySet.exclude(JavaCollectionConverter.asScalaSeq(Arrays.asList(excludedQueryIds)));
         }
-        return Arrays.asList(all);
+        final Division div = Division.parse(shard);
+        querySet = querySet.getShard(div.shard - 1, div.shardCount);
+        System.out.println("About to run queries: " + querySet.queryIds() + "... ");
+        return querySet;
       }
     };
   }
