@@ -22,48 +22,56 @@ import org.apache.spark.sql.{RunResult, SparkQueryRunner, SparkSession}
 
 import com.google.common.base.Preconditions
 import org.apache.commons.lang3.exception.ExceptionUtils
+import org.apache.hadoop.conf.Configuration
+import org.apache.hadoop.fs.{FileSystem, Path}
 
 import java.io.File
+import java.net.URI
 
-class QueryRunner(val queryResourceFolder: String, val dataPath: String) {
+class QueryRunner(val source: String, val dataPath: String) {
   import QueryRunner._
-
   Preconditions.checkState(
-    new File(dataPath).exists(),
+    fileExists(dataPath),
     s"Data not found at $dataPath, try using command `<gluten-it> data-gen-only <options>` to generate it first.",
     Array(): _*)
 
   def createTables(creator: TableCreator, spark: SparkSession): Unit = {
-    creator.create(spark, dataPath)
+    creator.create(spark, source, dataPath)
   }
 
   def runQuery(
       spark: SparkSession,
       desc: String,
-      caseId: String,
+      query: Query,
       explain: Boolean = false,
       sqlMetricMapper: MetricMapper = MetricMapper.dummy,
       executorMetrics: Seq[String] = Nil,
       randomKillTasks: Boolean = false): QueryResult = {
-    val path = "%s/%s.sql".format(queryResourceFolder, caseId)
     try {
       val r =
         SparkQueryRunner.runQuery(
           spark,
           desc,
-          path,
+          query,
           explain,
           sqlMetricMapper,
           executorMetrics,
           randomKillTasks)
-      println(s"Successfully ran query $caseId. Returned row count: ${r.rows.length}")
-      Success(caseId, r)
+      println(s"Successfully ran query ${query.id}. Returned row count: ${r.rows.length}")
+      Success(query.id, r)
     } catch {
       case e: Exception =>
-        println(s"Error running query $caseId. Error: ${ExceptionUtils.getStackTrace(e)}")
-        Failure(caseId, e)
+        println(s"Error running query ${query.id}. Error: ${ExceptionUtils.getStackTrace(e)}")
+        Failure(query.id, e)
     }
   }
+
+  private def fileExists(datapath: String): Boolean = {
+    if (datapath.startsWith("hdfs:") || datapath.startsWith("s3a:")) {
+      val uri = URI.create(datapath)
+      FileSystem.get(uri, new Configuration()).exists(new Path(uri.getPath))
+    } else new File(datapath).exists()
+  }
 }
 
 object QueryRunner {
