@@ -29,29 +29,36 @@ import java.sql.Date
 import scala.collection.JavaConverters._
 
 class TpchDataGen(
-    val spark: SparkSession,
+    spark: SparkSession,
     scale: Double,
     partitions: Int,
-    path: String,
-    typeModifiers: List[TypeModifier] = List())
+    source: String,
+    dir: String,
+    featureNames: Seq[String],
+    typeModifiers: Seq[TypeModifier])
   extends Serializable
   with DataGen {
 
+  private val featureRegistry = new DataGen.FeatureRegistry
+  private val features = featureNames.map(featureRegistry.getFeature)
+
   override def gen(): Unit = {
-    generate(path, "lineitem", lineItemSchema, partitions, lineItemGenerator, lineItemParser)
-    generate(path, "customer", customerSchema, partitions, customerGenerator, customerParser)
-    generate(path, "orders", orderSchema, partitions, orderGenerator, orderParser)
+    generate(dir, "lineitem", lineItemSchema, partitions, lineItemGenerator, lineItemParser)
+    generate(dir, "customer", customerSchema, partitions, customerGenerator, customerParser)
+    generate(dir, "orders", orderSchema, partitions, orderGenerator, orderParser)
     generate(
-      path,
+      dir,
       "partsupp",
       partSupplierSchema,
       partitions,
       partSupplierGenerator,
       partSupplierParser)
-    generate(path, "supplier", supplierSchema, partitions, supplierGenerator, supplierParser)
-    generate(path, "nation", nationSchema, nationGenerator, nationParser)
-    generate(path, "part", partSchema, partitions, partGenerator, partParser)
-    generate(path, "region", regionSchema, regionGenerator, regionParser)
+    generate(dir, "supplier", supplierSchema, partitions, supplierGenerator, supplierParser)
+    generate(dir, "nation", nationSchema, nationGenerator, nationParser)
+    generate(dir, "part", partSchema, partitions, partGenerator, partParser)
+    generate(dir, "region", regionSchema, regionGenerator, regionParser)
+
+    features.foreach(feature => DataGen.Feature.run(spark, source, feature))
   }
 
   // lineitem
@@ -336,7 +343,9 @@ class TpchDataGen(
           rows
       }(ShimUtils.getExpressionEncoder(modifiedSchema))
       .write
+      .format(source)
       .mode(SaveMode.Overwrite)
-      .parquet(dir + File.separator + tableName)
+      .option("path", dir + File.separator + tableName) // storage location
+      .saveAsTable(tableName)
   }
 }
